//---------------------------------------------------------------------------
// Here the FFT and the spectrum of the upper panel are computed. The waterfall
// is then displayed by the routine DoWF1();
// Also some functions are computed, like the computations for the fractional resampler
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "Functions.h"

#pragma package(smart_init)

	static double a1 = 0., a2 = 0., b0 = 0.;
	static double bdRight[RLEN];
	static float  bRight[RLEN], prefac, onehalf[2] = {0.5F, 0.5F}, one[2] = {1.F, 1.F};
		   float  resampfac;
		   float  resampfacStereo;
	static int    LeftOrder;
	static float  RightTaps[2*(RLEN+1)], LeftRing[LLEN], Dly[2*(RLEN+1)];
	static NSPFirState RightFDState;

	static bool   sw[5];
	static NSPFirState PS1astate, PS1bstate, PS1cstate, PS2astate, PS2bstate, PS2cstate,
					   PS3astate, PS3bstate, PS3cstate, PS4astate, PS4bstate, PS4cstate,
					   PS5astate, PS5bstate, PS5cstate;

//---------------------------------------------------------------------------
void __fastcall TProcessFF1::ComputeFF1IQ(void)      // called by ProcessFF1
{                                                    // when in IQ mode
    static int   direct, inverse, order, oldpolyfac = 0;
	int    j;
    float  tmp, dBcorr;

	if(size != MF->ff1Size || MainForm->polyfactor != oldpolyfac)
    {
        ZeroMemory(PolyQue,  sizeof(SCplx)*MainForm->polyfactor*FF1SMAX);
        nspcFreeTwdTbls();
        nspFreeBitRevTbls();

		size   = MF->ff1Size;
        order  = (int)(log(size)/log(2.) +0.0001);  // povero Borland....
        direct  = NSP_Forw | NSP_NoBitRev;
        inverse = NSP_Inv  | NSP_NoBitRev;
        nspcFft(NULL, order, (direct | NSP_Init));

// compute the sin^3 and sin^5 windows for the new size
        for(j = 0; j < MainForm->polyfactor * size; j++)
        {
		   tmp = sin((float)j / (MainForm->polyfactor * size-1) * PI);
           sin3[j].re = tmp * tmp * tmp;           sin3[j].im = 0;
		   sin5[j].re = sin3[j].re * tmp * tmp;    sin5[j].im = 0;
        }

// compute the Nuttall window for the new size
		for(j = 0; j < MainForm->polyfactor * size; j++)
		{
           wnuttall[j].re = 0.355768 - 0.487396 * cos(2*PI*j/(MainForm->polyfactor * size-1))
                        +0.144232  * cos(4*PI*j/(MainForm->polyfactor * size-1))
                        - 0.012604 * cos(6*PI*j/(MainForm->polyfactor * size-1));

           wnuttall[j].im = 0;
        }

        oldpolyfac = MainForm->polyfactor;
    }

    EnterCriticalSection(&MainForm->acFF1que);
	   memcpy(PolyFF1, PolyQue, MainForm->polyfactor * size * sizeof(SCplx));
	LeaveCriticalSection(&MainForm->acFF1que);

    switch(MainForm->wintype)
    {
       case Hann  :
			nspcWinHann(PolyFF1, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
            {
              case 1  : dBcorr = 1.0593;  break;
			  case 2  : dBcorr = 0.6683;  break;
			  case 4  : dBcorr = 0.9333;  break;
            }
			break;

       case Blackman  :
            nspcWinBlackman(PolyFF1, MainForm->polyfactor * size, -0.16);
            switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.2589;  break;
              case 2  : dBcorr = 0.7413;  break;
              case 4  : dBcorr = 0.7674;  break;
            }
            break;

       case Hamming  :
            nspcWinHamming(PolyFF1, MainForm->polyfactor * size);
            switch(MainForm->polyfactor)
			{
              case 1  : dBcorr = 1.0116;  break;
              case 2  : dBcorr = 0.6607;  break;
              case 4  : dBcorr = 1.1614;  break;
			}
            break;

       case wsin3  :
			nspcbMpy2(sin3, PolyFF1, MainForm->polyfactor * size);
            switch(MainForm->polyfactor)
			{
              case 1  : dBcorr = 1.2445;  break;
              case 2  : dBcorr = 0.7328;  break;
              case 4  : dBcorr = 0.7499;  break;
            }
            break;

       case wsin5  :
            nspcbMpy2(sin5, PolyFF1, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
            {
              case 1  : dBcorr = 1.5311;  break;
              case 2  : dBcorr = 0.8511;  break;
              case 4  : dBcorr = 0.6683;  break;
            }
            break;

       case nuttall :
            nspcbMpy2(wnuttall, PolyFF1, MainForm->polyfactor * size);
            switch(MainForm->polyfactor)
			{
              case 1  : dBcorr = 1.4454;  break;
			  case 2  : dBcorr = 0.8222;  break;
              case 4  : dBcorr = 0.6918;  break;
			}
			break;

	   case nowin  :
            switch(MainForm->polyfactor)
            {
			  case 1  : dBcorr = 0.5957;  break;
              case 2  : dBcorr = 0.5957;  break;
              case 4  : dBcorr = 0.5957;  break;
			}
            break;
    }

                          // WOLA processing
    memcpy(FF1array, PolyFF1, size * sizeof(SCplx)); // copy only the first chunk

    for(j = 1; j < MainForm->polyfactor; j++)
	  nspcbAdd2(PolyFF1 + j * size, FF1array, size);  // and add the remaining


    nspcFftNip(FF1array, FF1data, order, direct);

// compute magnitude : complex input; real output; single precision
    nspcbMag(FF1data,  FF1spectrum, size);

    nspsbMpy1(dBcorr, FF1spectrum, size);  // window correction factor

	DoWF1(FF1spectrum);

	return;
}
//---------------------------------------------------------------------------
void __fastcall TProcessFF1::ScrollWF1()
{
	if(MainForm->revWF1scroll)
	  MainForm->WF1bm->Canvas->CopyRect(MainForm->WF1src,
							   MainForm->WF1bm->Canvas, MainForm->WF1dest);
	else
	  MainForm->WF1bm->Canvas->CopyRect(MainForm->WF1dest,
							   MainForm->WF1bm->Canvas, MainForm->WF1src);
}
//---------------------------------------------------------------------------
void __fastcall TProcessFF1::PaintWF1()
{
	MainForm->Canvas->FrameRect(MainForm->FF1rect);
	MainForm->WF1->Repaint();
}
//---------------------------------------------------------------------------
// this is the NCO routine
// len is the buffer length to fill with sin and cos values
// freq is the frequency which must be generated
// srate is the sampling rate
// fac is both the amplitude (the absolute value of fac) and a switch that
//    indicates whether +sin must be generated (fac > 0) or -sin (fac < 0)
// phAdjust and ampAdjust specify a fine adjustment in amplitude and phase
//     (no adjustment -> ampAdjust = 1 and phAdjust = 0)
//---------------------------------------------------------------------------
void NCOGen(float *cosbuf, float *sinbuf, int len, float freq, float srate,
			float fac, float phAdjust, float ampAdjust, char which)
{
	static float LOphaseAcc, BFOphaseAcc, phaseAcc;
	int idx;

	float  absfac = fabs(fac);
	float ph, inc = (float)SINELEN * freq/srate;

	if(MainForm->justStarted) LOphaseAcc = BFOphaseAcc = 0;
	ph =  (fac > 0) ? 270. : 90.;
	ph += phAdjust;
	ph *= SINELEN/360.;

	switch(which)
	{
	   case 'L' :  phaseAcc = LOphaseAcc;  break;
	   case 'B' :  phaseAcc = BFOphaseAcc; break;
	}

	for(int k=0; k<len; k++)
	{
	   cosbuf[k] = MainForm->cosval[(int)phaseAcc] * absfac;
	   if(sinbuf != NULL)
	   {  idx = phaseAcc + ph;
		  if(idx >= SINELEN) idx -= SINELEN;
		  sinbuf[k] = MainForm->cosval[idx] * absfac * ampAdjust;
	   }
	   phaseAcc += inc;
	   if((int)phaseAcc >= SINELEN) phaseAcc -= SINELEN;
	   if((int)phaseAcc <  0)       phaseAcc += SINELEN;
	}

	if(which == 'L')
	  LOphaseAcc  = phaseAcc;
	else
	  BFOphaseAcc = phaseAcc;
}
//---------------------------------------------------------------------------
void __fastcall InitLO()
{
	MainForm->cosval = new float[SINELEN];

// compute cosine table
	for(int k=0; k < SINELEN; k++)
		MainForm->cosval[k] = cos(2. * PI * (float)k/SINELEN);
}
//---------------------------------------------------------------------------
// Here the wntire input segment is shifted in frequency, so that the LO frequency
// corresponds to zero Hz in the baseband
//---------------------------------------------------------------------------
void __fastcall DownConvert(SCplx samples[], SCplx Dsamp[])
{
   float   cosVal[BUFSIZE], sinVal[BUFSIZE], LOfreq;
   static  float prevFF1freq = 0.;

   MainForm->FF1freq += MainForm->freqerr;
   if(MainForm->FF1freq != prevFF1freq)
	  MainForm->AdjustBlueWindow();

   LOfreq = prevFF1freq = MainForm->FF1freq;
   if(MainForm->mode == CW)
	 LOfreq -= MainForm->FF2cf;

// generate the LO signal
   NCOGen(cosVal, sinVal, BUFSIZE, LOfreq, MainForm->srateIn, -1., 0., 1., 'L');

// prepare a complex vector for the mixer
   nspcb2RealToCplx(cosVal, sinVal, Dsamp, BUFSIZE);

// complex mixer
   nspcbMpy2(samples, Dsamp, BUFSIZE);
}
//---------------------------------------------------------------------------
// Here the fractional resampling is done, using the method dnsample of the class DN
// At the same time the resampling factor is computed, so that to keep stable
// the number of the queued output buffers
//---------------------------------------------------------------------------
bool __fastcall SincDecimate(SCplx samples[], SCplx Dsamp[])
{
   int    curlen, outlen;
   static int setpoint, passcount = 0;
   SCplx  tmp[BUFSIZE*2];
   static float  err, pvar, prevpvar, lolim, hilim;
   static bool locLock = true;

#define SETPOINT OUTBUFFERMARK


   if(MainForm->justStarted)
   {
	 locLock = true;
	 passcount = 0;
//	 pvar = prevpvar = MainForm->normQue;
	 pvar = prevpvar = (MainForm->srateIn < 1000000) ? SETPOINT : SETPOINT + 2;

	 resampfac = PreScaleInit(MainForm->downFactor);
	 lolim = resampfac * 0.9;
	 hilim = resampfac * 1.1;
	 lolim = resampfac * 0.998;	// AlpAdd
	 hilim = resampfac * 1.002;	// AlpAdd
	 setpoint = (MainForm->srateIn < 1000000) ? SETPOINT : SETPOINT + 2;
	 MainForm->Dn->setDnfactor(resampfac);
	 err = 0.; //prevpvar = SETPOINT;
   }

   if(restarted && !MainForm->wavin)
   {
	  float div, savdec = resampfac;

	  if(passcount++ > 100)
	  {
		pvar = (MainForm->normQue + 9.F * pvar) / 10.F;
		err  = (pvar - prevpvar) / ((1.e6 * prefac)/ MainForm->srateIn);
		prevpvar = pvar;
		resampfac += err;
		// err = (pvar - setpoint) / 12000.F;
		err = (pvar - setpoint) / (double(MainForm->srateOut) / 2.);
		resampfac += err;
		locLock &= (pvar >= setpoint-2 && pvar <= setpoint+4);

		if(resampfac < 1.)    resampfac = 1.;
		if(resampfac < lolim) resampfac = lolim;
		if(resampfac > hilim) resampfac = hilim;
		if(MainForm->declock || locLock || MainForm->mode == DRM)
			resampfac = savdec;

		if(fabs(pvar - (float)setpoint) < 0.1F)
			resampfac = savdec;
	  }
	  MainForm->Dn->setDnfactor(resampfac);
   }

// now do the prescaling of the sampling frequency

   if(!PreScale(samples))
	 return false;        // return false if a BUFSIZE long buffer is not ready yet

// now do the actual fractional resampling
   outlen = MainForm->Dn->dnsample(samples, tmp, BUFSIZE);

   MainForm->Decring->put(tmp, outlen);
   curlen = MainForm->Decring->getlen();

   if(curlen < BUFSIZE)
	 return false;        // return false if a BUFSIZE long buffer is not ready yet

   MainForm->Decring->get(Dsamp, BUFSIZE);
   return true;
}


//---------------------------------------------------------------------------
// resamp function used to resample the vector of the indexes of the passband
// filter in SP2 for the various resolutions
//---------------------------------------------------------------------------
void __fastcall resamp(int bef, int aft, int *buff)
{
   int ct, i, k, offs;

   if(bef>=aft)
   {
	 offs=bef/(2*aft);
	 for(ct=i=0; i<bef; i++)
		if((ct+=aft) >= bef) {ct-=bef;*(buff++)=i-offs;}
   }
   else
   {
	 for(ct=k=i=0; i< aft; i++)
	 {
		if((ct+=bef) >  aft) {k++; ct-=aft;}
		*(buff++)=k;
	 }
   }
   return;
}
//---------------------------------------------------------------------------
// DoPEak and DoPeak2 implement the CW peaking function
//---------------------------------------------------------------------------
void __fastcall TMainForm::DoPeak(float *X)
{
   static double y0, y1=0., y2=0., x0;

// IIR resonator with two zeros in the origin and two poles in wr and -wr
   for (int k = 0; k < BUFSIZE; k++)
   {  x0 = X[k];
	  y0 = -a1 * y1 - a2 * y2 + b0 * x0;
	  y2 = y1;
	  y1 = y0;
	  X[k] = y0;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DoPeak2(float *X)
{
   static double y0, y1=0., y2=0., x0;

// IIR resonator with two zeros in the origin and two poles in wr and -wr
   for (int k = 0; k < BUFSIZE; k++)
   {  x0 = X[k];
	  y0 = -a1 * y1 - a2 * y2 + b0 * x0;
	  y2 = y1;
	  y1 = y0;
	  X[k] = y0;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Compute_IIR_parms(void)
{
   double r, r2, wr, cosw0, corr;

   r = Den2qfac;

   a1 = a2 = b0 = 0.;  // in case of parallel thread execution
   r2 = r*r;
   wr = 2 * CWPitch / vsrOut * PI;
   corr = PI - wr; // gain correction empirically confirmed with Matlab
   cosw0 = (2 * r / (1 + r2)) * cos(wr); // resonance frequency correction
										 // (see the Proakis & Manolakis book)

   a1 = -2. * r * cosw0;
   a2 = r2;
// b0 is normalized for gain = 0dB on all the band
   b0 = corr * (1. - r) * sqrt(1. + r2 - 2 * r * cosw0);
}
//---------------------------------------------------------------------------
// computes a FIR filter using the windowed sync method (see the Laakso paper
// and also my Matlab function wsinc.m). The input is the order N and fractional
// delay del, computed from the skew variable. Output is in the bRight array of
// coefficients.
//---------------------------------------------------------------------------
void __fastcall Wsinc(void)
{

	int k, n;
	const int N = ALLPASSLENGTH, D = (ALLPASSLENGTH - 1)/2;
	float del, n_minusD, Dtot;

	del = MainForm->skew / (1.E6 / (float)MainForm->srateIn);
	if(del >= 0)
	{
	  if(del <= 0.5) LeftOrder = D;
	  else
	  {
		 LeftOrder = D - 1;
		 del = del - 1;
	  }
	}
	else  // del < 0
	{
	  if(del > -0.5) LeftOrder = D;
	  else
	  {
		 LeftOrder = D + 1;
		 del = del + 1;
	  }
	}

	Dtot = D + del;
	for(n=0; n<=N; n++)
	{
	  n_minusD = PI * (n - Dtot);
	  if(fabs(n_minusD) < 1.e-80) bRight[n] = 1.;  // sinc(0) == 1 !
	  else                  bRight[n] = sin(n_minusD) / n_minusD;
	}

	nspsWinBlackman(bRight, N+1, -0.16);

	EnterCriticalSection(&MainForm->FDupdate);
	  if(RightFDState.tapsLen > 0)
	  { nspsFirGetDlyl(&RightFDState, Dly);
		nspFirFree(&RightFDState);
	  }
	  nspsFirInit(bRight, N+1, Dly, &RightFDState);
	LeaveCriticalSection(&MainForm->FDupdate);
}
//---------------------------------------------------------------------------
// this the fractional delay routine. The left channel is delayed by a fixed
// amount, while the right channel is delayed by a fractional amount,
// using the coefficients computed by the Wsinc routine
//---------------------------------------------------------------------------
void __fastcall FDFilter(float* left, float* right)
{
	float save, rightout[BUFSIZE];
	static int idx = 0;

	EnterCriticalSection(&MainForm->FDupdate);
	  nspsbFir(&RightFDState, right, rightout, BUFSIZE);
	LeaveCriticalSection(&MainForm->FDupdate);

	memcpy(right, rightout, sizeof(float) * BUFSIZE);

// this is the fixed delay on the left channel
	for(int k=0; k<BUFSIZE; k++)
	{
	  save = LeftRing[idx];
	  LeftRing[idx++] = left[k];
	  left[k] = save;
	  if(idx >= LeftOrder) idx = 0;
	}
}
//---------------------------------------------------------------------------
// Here the prescaler that is in front of the fractional resampler is initialized
//---------------------------------------------------------------------------
float __fastcall PreScaleInit(float downfactor)
{
	int k;
	static bool firinited = false;
	static int  previousrate = 0;

	//
	// AlpAdd - BEGIN
	k = 1 << int(log(MainForm->srateIn / float(MainForm->srateOut) / 1.5) / log(2.));
	// for(k = 32; k >= 1; k/=2)
	//    if(downfactor / (float)k > 1.5F) break;
	prefac = float(k);
	for(int n = 0; n < 5; n++)  sw[n] = (k > (1 << n)) ? true : false;

	/*
	for(int n = 0; n < 5; n++)  sw[n] = false;

	switch(k)
	{
		case 32 : sw[0] = sw[1] = sw[2] = sw[3] = sw[4] = true;
												prefac = 32.F;break;  // prescaled by 32

		case 16 : sw[0] = sw[1] = sw[2] = sw[3] = true;
												prefac = 16.F;break;  // prescaled by 16

		case 8  : sw[0] = sw[1] = sw[2] = true; prefac = 8.F; break;  // prescaled by 8
		case 4  : sw[0] = sw[1] = true;         prefac = 4.F; break;  // prescaled by 4
		case 2  : sw[0] = true;                 prefac = 2.F; break;  // prescaled by 2
		case 1  : case 0 :                      prefac = 1.F; break;  // prescaled by 1
	}
	*/
	// AlpAdd - END
	//

	if(MainForm->srateIn != previousrate)
	{
		if(previousrate != 0)
		{
			nspFirFree(&PS1astate);
			nspFirFree(&PS1bstate);
			nspFirFree(&PS1cstate);
			nspFirFree(&PS2astate);
			nspFirFree(&PS2bstate);
			nspFirFree(&PS2cstate);
			nspFirFree(&PS3astate);
			nspFirFree(&PS3bstate);
			nspFirFree(&PS3cstate);
			nspFirFree(&PS4astate);
			nspFirFree(&PS4bstate);
			nspFirFree(&PS4cstate);
			nspFirFree(&PS5astate);
			nspFirFree(&PS5bstate);
			nspFirFree(&PS5cstate);
		}
		previousrate = MainForm->srateIn;
		firinited = false;
	}

	if(!firinited)  // init a bunch of boxcar filters (N = 2)
	{
		firinited = true;
		nspcsFirInit(onehalf, 2, NULL, &PS1astate);
		nspcsFirInit(onehalf, 2, NULL, &PS1bstate);
		nspcsFirInit(onehalf, 2, NULL, &PS1cstate);

		nspcsFirInit(onehalf, 2, NULL, &PS2astate);
		nspcsFirInit(onehalf, 2, NULL, &PS2bstate);
		nspcsFirInit(onehalf, 2, NULL, &PS2cstate);

		nspcsFirInit(onehalf, 2, NULL, &PS3astate);
		nspcsFirInit(onehalf, 2, NULL, &PS3bstate);
		nspcsFirInit(onehalf, 2, NULL, &PS3cstate);

		nspcsFirInit(onehalf, 2, NULL, &PS4astate);
		nspcsFirInit(onehalf, 2, NULL, &PS4bstate);
		nspcsFirInit(onehalf, 2, NULL, &PS4cstate);

		nspcsFirInit(onehalf, 2, NULL, &PS5astate);
		nspcsFirInit(onehalf, 2, NULL, &PS5bstate);
		nspcsFirInit(onehalf, 2, NULL, &PS5cstate);
	}


	return downfactor / prefac;
}


//---------------------------------------------------------------------------
// Here the prescaling is actually done
//---------------------------------------------------------------------------
bool __fastcall PreScale(SCplx *samples)
{
	float   pb, hi, lo, dist, cutoff;
	static  float oldcutoff = 0.F, LOfreq;
	static SCplx ya[BUFSIZE],    yb[BUFSIZE],    yc[BUFSIZE],
				 yd[BUFSIZE/2],  ye[BUFSIZE/2],  yf[BUFSIZE/2],
				 yg[BUFSIZE/4],  yh[BUFSIZE/4],  yi[BUFSIZE/4],
				 yj[BUFSIZE/8],  yk[BUFSIZE/8],  yl[BUFSIZE/8],
				 ym[BUFSIZE/16], yn[BUFSIZE/16], yo[BUFSIZE/16],  y[BUFSIZE];

	int k, len;

	SCplx const *tptr_i;
	SCplx *tptr_o = y;

	if(sw[0])
	{
		tptr_o = y;
		if(MainForm->srateIn > 500000 && MainForm->mode == FM && MainForm->submode != lower)
		{
			tptr_i = samples;
		}
		else
		{
			nspcsbFir(&PS1astate, samples, ya, BUFSIZE);  // boxcar filter (N = 2)
			nspcsbFir(&PS1bstate, ya,      yb, BUFSIZE);  // boxcar filter (N = 2)
			nspcsbFir(&PS1cstate, yb,      yc, BUFSIZE);  // boxcar filter (N = 2)
			tptr_i = yc;
		}
		for(k = 0; k < BUFSIZE; k+=2)
		{
			// y[k/2] = yc[k];    // decimate by 2
			*tptr_o++ = *tptr_i++;
			tptr_i++;
		}

		len = BUFSIZE/2;
	}

	if(sw[1])
	{
		tptr_o = y;
		if(MainForm->srateIn > 1000000 && MainForm->mode == FM && MainForm->submode != lower)
		{
			tptr_i = y;
			// for(k = 0; k < BUFSIZE/2; k+=2) y[k/2] = y[k];    // decimate by 2
		}
		else
		{
			nspcsbFir(&PS2astate, y,  yd, BUFSIZE/2);     // boxcar filter (N = 2)
			nspcsbFir(&PS2bstate, yd, ye, BUFSIZE/2);     // boxcar filter (N = 2)
			nspcsbFir(&PS2cstate, ye, yf, BUFSIZE/2);     // boxcar filter (N = 2)
			tptr_i = yf;
			// for(k = 0; k < BUFSIZE/2; k+=2) y[k/2] = yf[k];    // decimate by 2
		}

		for(k = 0; k < BUFSIZE; k+=2)
		{
			*tptr_o++ = *tptr_i++;
			tptr_i++;
		}

		len = BUFSIZE/4;
	}

	if(sw[2])
	{
		tptr_o = y;
		if(MainForm->srateIn >= 2000000 && MainForm->mode == FM && MainForm->submode != lower)
		{
			tptr_i = y;
		}
		else
		{
			nspcsbFir(&PS3astate, y,  yg, BUFSIZE/4);     // boxcar filter (N = 2)
			nspcsbFir(&PS3bstate, yg, yh, BUFSIZE/4);     // boxcar filter (N = 2)
			nspcsbFir(&PS3cstate, yh, yi, BUFSIZE/4);     // boxcar filter (N = 2)
			tptr_i = yi;
		}
		// for(k = 0; k < BUFSIZE/4; k+=2) y[k/2] = yi[k];    // decimate by 2

		for(k = 0; k < BUFSIZE; k+=2)
		{
			*tptr_o++ = *tptr_i++;
			tptr_i++;
		}

		len = BUFSIZE/8;
	}

	if(sw[3])
	{
		nspcsbFir(&PS4astate, y,  yj, BUFSIZE/8);     // boxcar filter (N = 2)
		nspcsbFir(&PS4bstate, yj, yk, BUFSIZE/8);     // boxcar filter (N = 2)
		nspcsbFir(&PS4cstate, yk, yl, BUFSIZE/8);     // boxcar filter (N = 2)

		// for(k = 0; k < BUFSIZE/8; k+=2) y[k/2] = yl[k];    // decimate by 2

		tptr_i = yl;
		tptr_o = y;
		for(k = 0; k < BUFSIZE; k+=2)
		{
			*tptr_o++ = *tptr_i++;
			tptr_i++;
		}

		len = BUFSIZE/16;
	}

	if(sw[4])
	{
		nspcsbFir(&PS5astate, y,  ym, BUFSIZE/16);     // boxcar filter (N = 2)
		nspcsbFir(&PS5bstate, ym, yn, BUFSIZE/16);     // boxcar filter (N = 2)
		nspcsbFir(&PS5cstate, yn, yo, BUFSIZE/16);     // boxcar filter (N = 2)

		// for(k = 0; k < BUFSIZE/16; k+=2) y[k/2] = yo[k];    // decimate by 2

		tptr_i = yo;
		tptr_o = y;
		for(k = 0; k < BUFSIZE; k+=2)
		{
			*tptr_o++ = *tptr_i++;
			tptr_i++;
		}

		len = BUFSIZE/32;
	}

	if(!sw[0] && !sw[1] && !sw[2] && !sw[3] && !sw[4]) return true;

	MainForm->PSring->put(y, len);

	if(MainForm->PSring->getlen() < BUFSIZE)
	  return false;

	MainForm->PSring->get(y, BUFSIZE);

// compute a cutoff freq. for the filter, which is the minimum between half
// of the out sample rate, and the distance of the LO from the input Nyquist (prescaled)
// (cutoff ranges from 0 to 0.5)

   LOfreq = MainForm->FF1freq;
   if(MainForm->mode == CW)
	 LOfreq -= MainForm->FF2cf;

   hi = MainForm->srateIn/2.F;
   lo = (MainForm->datamode == mIQ) ? -hi : 0.;
   dist = (MainForm->mode == LSB) ? LOfreq - lo : hi - LOfreq;
   pb = min(MainForm->vsrOut * 0.46, dist);   // 0.92 Nyquist .....

   cutoff     = max(0.001F, pb / ((float)MainForm->srateIn / prefac));

   if(oldcutoff != cutoff) // then compute the new anti-alias filter taps
   {
	 if(NSP_StsOk != nspdFirLowpass(cutoff, dtapValsLO, LPFIRLEN,
												   NSP_WinBlackmanOpt, 1))
	 { fatal("An error occurred when computing the anti-alias filter");
	   Application->Terminate();
	 }

	 for(int k = 0; k < LPFIRLEN; k++) ftapValsLO[k] = dtapValsLO[k];

	 if(oldcutoff != 0)  // if at least one time through before...
	   nspFirFree(&MainForm->statePtrLO);

	 oldcutoff = cutoff;
	 nspcsFirInit(ftapValsLO, LPFIRLEN, NULL, &MainForm->statePtrLO);
   }

// and now finally lowpass the prescaled samples, before the final fractional resampling
	 nspcsbFir(&MainForm->statePtrLO, y, samples, BUFSIZE);

	return true;
}
//---------------------------------------------------------------------------
SCplx __inline mycAdd(SCplx a, SCplx b)
{
	SCplx ret;

	ret.re = a.re + b.re;
	ret.im = a.im + b.im;

	return ret;
}
//---------------------------------------------------------------------------
SCplx __inline mycAvg(SCplx a, SCplx b)
{
	SCplx ret;

	ret.re = (a.re + b.re)/2.F;
	ret.im = (a.im + b.im)/2.F;

	return ret;
}
//---------------------------------------------------------------------------
SCplx __inline mycMpy(SCplx a, SCplx b)
{
	SCplx ret;

	ret.re = a.re * b.re - a.im * b.im;
	ret.im = a.re * b.im + a.im * b.re;

	return ret;
}
//---------------------------------------------------------------------------
SCplx __inline mycAbs(SCplx a)
{
	SCplx ret;

	ret.re = fabs(a.re);
	ret.im = fabs(a.im);

	return ret;
}
//---------------------------------------------------------------------------
