//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "PreFF1Proc.h"

//---------------------------------------------------------------------------
// This routine handles the raw data from the input source, before the first FFT
// is computed. It is used for the(still experimental) despreader function
//---------------------------------------------------------------------------
void __fastcall PreFF1Proc(SCplx *LR)
{
	int    idx, loIdx, hiIdx, span;
	float  ProcSpec[PREFFTLEN], ba, ke, th, HzBin;
	SCplx  FF1Proc[PREFFTLEN];

	static float base, kern, keprev, thresh, Fac;
	static bool  first = true, prev = false;;
	static int   direct, inverse, size, order, prevLo = 1.e6, prevHi = 1.e6;
	static SCplx TimeSeries[PREFFTLEN], Mask[PREFFTLEN];

	switch(MainForm->mode)
	{
	  case USB :
		loIdx = MainForm->FF2locut + MainForm->FF1freq + MainForm->srateIn/2 + 0.5;
		hiIdx = MainForm->FF2hicut + MainForm->FF1freq + MainForm->srateIn/2 + 0.5;
		break;

	  case LSB :
		loIdx = -MainForm->FF2hicut + MainForm->FF1freq + MainForm->srateIn/2 + 0.5;
		hiIdx = -MainForm->FF2locut + MainForm->FF1freq + MainForm->srateIn/2 + 0.5;
		break;

	  case CW  :
		int hspan = (MainForm->FF2hicut - MainForm->FF2locut) / 2;
		loIdx = MainForm->FF1freq - hspan + MainForm->srateIn/2 + 0.5;
		hiIdx = MainForm->FF1freq + hspan + MainForm->srateIn/2 + 0.5;
		break;
	}

	loIdx = loIdx * (float)PREFFTLEN / MainForm->srateIn + 0.5;
	hiIdx = hiIdx * (float)PREFFTLEN / MainForm->srateIn + 0.5;

	loIdx += PREFFTLEN/2;
	if(loIdx > PREFFTLEN) loIdx -= PREFFTLEN; if(loIdx < 0) loIdx = 0;

	hiIdx += PREFFTLEN/2;
	if(hiIdx > PREFFTLEN) hiIdx -= PREFFTLEN; if(hiIdx < 0) hiIdx = 0;

	if(loIdx != prevLo || hiIdx != prevHi)
	{
	  prevLo = loIdx;    prevHi = hiIdx;

	  if(loIdx < hiIdx)
		for(int k = 0; k < PREFFTLEN; k++)
		  if(k < loIdx || k > hiIdx)
		  { Mask[k].re = 0.;    Mask[k].im = 0.; }
		  else if(k == loIdx || k == hiIdx)
		  { Mask[k].re = 0.5;  Mask[k].im = 0.5; }
		  else
		  { Mask[k].re = 1.;  Mask[k].im = 1.; }
	  else
		for(int k = 0; k < PREFFTLEN; k++)
		  if(k > hiIdx && k < loIdx)
		  { Mask[k].re = 0.;    Mask[k].im = 0.; }
		  else if(k == loIdx || k == hiIdx)
		  { Mask[k].re = 0.5;  Mask[k].im = 0.5; }
		  else
		  { Mask[k].re = 1.;  Mask[k].im = 1.; }
	}

// init FFT and other things
	if(first)
	{
	  first = false;
	  prev  = false;
	  base  = kern = keprev = thresh = 0.;
	  nspzFreeTwdTbls();
	  nspFreeBitRevTbls();

	  size    = PREFFTLEN;
	  order   = (int)(log(size)/log(2.) +0.0001); // povero Borland....
	  direct  = NSP_Forw | NSP_NoBitRev;
	  inverse = NSP_Inv  | NSP_NoBitRev;
	  nspzFft(NULL, order, (direct | NSP_Init));
	}

	memcpy(TimeSeries, TimeSeries + BUFSIZE, sizeof(SCplx) * (PREFFTLEN - BUFSIZE));
	memcpy(TimeSeries + (PREFFTLEN - BUFSIZE), LR, sizeof(SCplx) * BUFSIZE);

//    nspcWinHamming(TimeSeries, size);
	nspcFftNip(TimeSeries, FF1Proc, order, direct);
	nspcbMag(FF1Proc, ProcSpec, size);

	span = hiIdx - loIdx;  if(span < 0) span += PREFFTLEN;

	if(span < 4 || !MainForm->despreading) goto no_despread;

// do despreading here
	ba = 0.;
	for(int k=loIdx; k<loIdx + span/4; k++)
	  ba += ProcSpec[k];

	for(int k=loIdx + 3*span/4; k<hiIdx; k++)
	  ba += ProcSpec[k];

	kern = 0.;
	for(int k=loIdx + span/4; k<loIdx + 3*span/4; k++)
	  if(kern < ProcSpec[k]) kern = ProcSpec[k];

	ba /= span/2;
	base = base * 0.5 + ba * 0.5;
	kern = (kern + 2. * keprev) / 3.;
	keprev = kern;
	th = (base + kern) / 2.;
	thresh = (thresh * 199. + th) / 200.;

	if(kern > thresh)
	  if(prev) MainForm->gt = true;
	  else
	  {
		prev = true;
		MainForm->gt = false;
	  }
	else
	  MainForm->gt = prev = false;


	for(int k=0; k<PREFFTLEN; k++)
	  { FF1Proc[k].re *= 1.e-3;  FF1Proc[k].im *= 1.e-3; }

	HzBin = MainForm->srateIn / PREFFTLEN;

	idx = MainForm->FF1freq / HzBin + 0.5;
	Fac = 1.; // * kern / base;
	if(MainForm->gt)
	{
//	  Swap(FF1Proc, PREFFTLEN);
	  FF1Proc[idx].re = Fac;
	  FF1Proc[idx].im = Fac;
//	  Swap(FF1Proc, PREFFTLEN);
	}


// ----------- end of despreading

no_despread:

	nspcFft(FF1Proc, order, inverse);
	memcpy(LR, FF1Proc + PREFFTLEN/2 - BUFSIZE/2, sizeof(SCplx) * BUFSIZE);
//	memcpy(LR, FF1Proc + PREFFTLEN - BUFSIZE, sizeof(SCplx) * BUFSIZE);

	return;
}
//---------------------------------------------------------------------------



#pragma package(smart_init)
