//---------------------------------------------------------------------------
// The implementation of some ring buffers classes
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "Ring.h"
#include "main.h"

//---------------------------------------------------------------------------
__fastcall TFRing::TFRing(int numel)
{
   size = numel;
   ring = new float[size];
   for(int k = 0; k < size; k++) ring[k] = 0.;
   Top = Que = 0;
}
//---------------------------------------------------------------------------
__fastcall TFRing::~TFRing()
{
   delete[] ring;
}
//---------------------------------------------------------------------------
bool __fastcall TFRing::get(float* el, int numel)
{
	if(numel > getlen()) exit(1);
   headroom = size - Que;

   if(numel <= headroom)
   {
     memmove(el, ring + Que, numel * sizeof(float));
   }
   else
   {
     memmove(el, ring + Que, headroom * sizeof(float));
     memmove(el + headroom, ring, (numel - headroom) * sizeof(float));
   }

   Que = (Que + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TFRing::put(float* el, int numel)
{
	if(numel > size - getlen()) exit(1);
   headroom = size - Top;

   if(numel <= headroom)
   {
     memmove(ring + Top, (float *)el, numel * sizeof(float));
   }
   else
   {
     memmove(ring + Top, (float *)el, headroom * sizeof(float));
     memmove(ring, (float *)el + headroom, (numel - headroom) * sizeof(float));
   }

   Top = (Top + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
int  __fastcall TFRing::getlen(void)
{
   int numel = Top - Que;
   if(numel < 0) numel += size;

   return numel;
}
//---------------------------------------------------------------------------
__fastcall TDRing::TDRing(int numel)
{
   size = numel;
   ring = new double[size];
   for(int k = 0; k < size; k++) ring[k] = 0.;
   Top = Que = 0;
}
//---------------------------------------------------------------------------
__fastcall TDRing::~TDRing()
{
   delete[] ring;
}
//---------------------------------------------------------------------------
bool __fastcall TDRing::get(double* el, int numel)
{
	if(numel > getlen()) exit(1);
   headroom = size - Que;

   if(numel <= headroom)
   {
	 memmove(el, ring + Que, numel * sizeof(double));
   }
   else
   {
	 memmove(el, ring + Que, headroom * sizeof(double));
	 memmove(el + headroom, ring, (numel - headroom) * sizeof(double));
   }

   Que = (Que + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TDRing::put(double* el, int numel)
{
	if(numel > size - getlen()) exit(1);
   headroom = size - Top;

   if(numel <= headroom)
   {
	 memmove(ring + Top, el, numel * sizeof(double));
   }
   else
   {
	 memmove(ring + Top, el, headroom * sizeof(double));
	 memmove(ring, el + headroom, (numel - headroom) * sizeof(double));
   }

   Top = (Top + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
int  __fastcall TDRing::getlen(void)
{
   int numel = Top - Que;
   if(numel < 0) numel += size;

   return numel;
}
//---------------------------------------------------------------------------
__fastcall TIRing::TIRing(int numel)
{
   size = numel;
   ring = new int[size];
   clear();
}
//---------------------------------------------------------------------------
__fastcall TIRing::~TIRing()
{
   delete[] ring;
}
//---------------------------------------------------------------------------
bool __fastcall TIRing::get(int* el, int numel)
{
	if(numel > getlen()) exit(1);
   headroom = size - Que;

   if(numel <= headroom)
   {
	 memmove(el, ring + Que, numel * sizeof(int));
   }
   else
   {
	 memmove(el, ring + Que, headroom * sizeof(int));
	 memmove(el + headroom, ring, (numel - headroom) * sizeof(int));
   }

   Que = (Que + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TIRing::put(int* el, int numel)
{
	if(numel > size - getlen()) exit(1);
   headroom = size - Top;

   if(numel <= headroom)
   {
	 memmove(ring + Top, el, numel * sizeof(int));
   }
   else
   {
	 memmove(ring + Top, el, headroom * sizeof(int));
	 memmove(ring, el + headroom, (numel - headroom) * sizeof(int));
   }

   Top = (Top + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
int  __fastcall TIRing::getlen(void)
{
   int numel = Top - Que;
   if(numel < 0) numel += size;

   return numel;
}
//---------------------------------------------------------------------------
void __fastcall TIRing::clear(void)
{
   for(int k = 0; k < size; k++) ring[k] = 0;
   Top = Que = 0;
}
//---------------------------------------------------------------------------
__fastcall TCRing::TCRing(int numel)
{
   size = numel;
   ring = new SCplx[size];
   for(int k = 0; k < size; k++) ring[k].re = ring[k].im = 0.;
   Top = Que = 0;
}
//---------------------------------------------------------------------------
__fastcall TCRing::~TCRing()
{
   delete[] ring;
}
//---------------------------------------------------------------------------
bool __fastcall TCRing::get(SCplx* el, int numel)
{
	if(numel > getlen()) exit(1);
   headroom = size - Que;

   if(numel <= headroom)
   {
	 memmove(el, ring + Que, numel * sizeof(SCplx));
   }
   else
   {
	 memmove(el, ring + Que, headroom * sizeof(SCplx));
	 memmove(el + headroom, ring, (numel - headroom) * sizeof(SCplx));
   }

   Que = (Que + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TCRing::put(SCplx* el, int numel)
{

	if(numel > size - getlen()) exit(1);
   headroom = size - Top;

   if(numel <= headroom)
   {
	 memmove(ring + Top, el, numel * sizeof(SCplx));
   }
   else
   {
	 memmove(ring + Top, el, headroom * sizeof(SCplx));
	 memmove(ring, el + headroom, (numel - headroom) * sizeof(SCplx));
   }

   Top = (Top + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
int  __fastcall TCRing::getlen(void)
{
   int numel = Top - Que;
   if(numel < 0) numel += size;

   return numel;
}
//---------------------------------------------------------------------------
__fastcall TVRing::TVRing(int numel, int elementsize)
{
   size = numel;
   elsize = elementsize;
   ring = new char[size * elsize];
   for(int k = 0; k < size * elsize; k++) ring[k] = 0;
   Top = Que = 0;
}
//---------------------------------------------------------------------------
__fastcall TVRing::~TVRing()
{
   delete[] ring;
}
//---------------------------------------------------------------------------
bool __fastcall TVRing::get(char* el, int numel)
{
	if(numel > getlen()) exit(1);
   headroom = size - Que;

   if(numel <= headroom)
   {
	 memmove(el, ring + Que * elsize, numel * elsize);
   }
   else
   {
	 memmove(el, ring + Que * elsize, headroom * elsize);
	 memmove(el + headroom * elsize, ring, (numel - headroom) * elsize);
   }

   Que = (Que + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TVRing::put(char* el, int numel)
{
	if(numel > size - getlen()) exit(1);
   headroom = size - Top;

   if(numel <= headroom)
   {
	 memmove(ring + Top * elsize, el, numel * elsize);
   }
   else
   {
	 memmove(ring + Top * elsize, el, headroom * elsize);
	 memmove(ring, el + headroom * elsize, (numel - headroom) * elsize);
   }

   Top = (Top + numel) % size;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
int  __fastcall TVRing::getlen(void)
{
   int numel = (Top - Que);
   if(numel < 0) numel += size;

   return numel;
}
//---------------------------------------------------------------------------
__fastcall TIQFifo::TIQFifo(int num, int length)
{
   numel = num;
   len = length;
   fifo = (short *)VirtualAlloc(NULL, numel * len * 2 * sizeof(short), MEM_COMMIT,
					   PAGE_READWRITE);
   VirtualLock(fifo, numel * len * 2 * sizeof(short));
   Top = Que = fifo;
}
//---------------------------------------------------------------------------
__fastcall TIQFifo::~TIQFifo()
{
   VirtualUnlock(fifo, numel * len * 2 * sizeof(short));
   VirtualFree(fifo, 0, MEM_RELEASE);
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo::getp(short* &el)
{
   el = Que;
   Que += len * 2;
   if(Que >= fifo + numel * len * 2) Que = fifo;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo::put(short* el)
{
   memcpy(Top, el, len * 2 * sizeof(short));
   Top += len * 2;

   if(Top >= fifo + numel * len * 2) Top = fifo;

   return true;
}
//---------------------------------------------------------------------------
int  __fastcall TIQFifo::getnum(void)
{
   int tmp = (Top - Que) / (len * 2);
   if(tmp < 0)
	 tmp += numel;
   return tmp;  // return number of elements, not length
}
//---------------------------------------------------------------------------
__fastcall TIQFifo24::TIQFifo24(int num, int length)
{
   numel = num;
   len = length;

   fifo = (char *)VirtualAlloc(NULL, numel * len * 2 * 3, MEM_COMMIT,
					   PAGE_READWRITE);
   VirtualLock(fifo, numel * len * 2 * 3);
   Top = Que = fifo;
}
//---------------------------------------------------------------------------
__fastcall TIQFifo24::~TIQFifo24()
{
   VirtualUnlock(fifo, numel * len * 2 * 3);   //  * sizeof(byte) omitted...
   VirtualFree(fifo, 0, MEM_RELEASE);
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo24::getp(void* &el)
{
   EnterCriticalSection(&MainForm->FifoAccess);
	 el = Que;
	 Que += len * 2 * 3;
	 if(Que >= fifo + numel * len * 2 * 3) Que = fifo;

   LeaveCriticalSection(&MainForm->FifoAccess);
   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo24::put(void* el)
{
   EnterCriticalSection(&MainForm->FifoAccess);
	 memcpy(Top, (char *)el, len * 2 * 3);
	 Top += len * 2 * 3;

	 if(Top >= fifo + numel * len * 2 * 3) Top = fifo;

   LeaveCriticalSection(&MainForm->FifoAccess);
   return true;
}
//---------------------------------------------------------------------------
int  __fastcall TIQFifo24::getnum(void)
{
   int tmp = (Top - Que) / (len * 2 * 3);
   if(tmp < 0)
	 tmp += numel;
   return tmp;  // return number of elements, not length
}
//---------------------------------------------------------------------------
__fastcall TIQFifo32::TIQFifo32(int num, int length)
{
   numel = num;
   len = length;

   fifo = (char *)VirtualAlloc(NULL, numel * len * 2 * 4, MEM_COMMIT,
					   PAGE_READWRITE);
   VirtualLock(fifo, numel * len * 2 * 4);
   Top = Que = fifo;
}
//---------------------------------------------------------------------------
__fastcall TIQFifo32::~TIQFifo32()
{
   VirtualUnlock(fifo, numel * len * 2 * 4);   //  * sizeof(byte) omitted...
   VirtualFree(fifo, 0, MEM_RELEASE);
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo32::getp(void* &el)
{
   el = Que;
   Que += len * 2 * 4;
   if(Que >= fifo + numel * len * 2 * 4) Que = fifo;

   return true;    // error return to be done (overrun and underrun)
}
//---------------------------------------------------------------------------
bool __fastcall TIQFifo32::put(void* el)
{
   memcpy(Top, (char *)el, len * 2 * 4);
   Top += len * 2 * 4;

   if(Top >= fifo + numel * len * 2 * 4) Top = fifo;

   return true;
}
//---------------------------------------------------------------------------
int  __fastcall TIQFifo32::getnum(void)
{
   int tmp = (Top - Que) / (len * 2 * 4);
   if(tmp < 0)
	 tmp += numel;
   return tmp;  // return number of elements, not length
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
