//---------------------------------------------------------------------------
// This is the implementationof the WavIO class, that does all the WAV file IO
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "Wave_io.h"

const int daymonth[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

//---------------------------------------------------------------------------
void WavIO::InRead(void *ptrL, void *ptrR, int& wreadlen)
{
	int cnt, i, k, readlen;
	__int64 secelapsed, tmp;
	unsigned char buf[32768], *p, s[256];
	AnsiString newname;

tryagain:
	readlen = wreadlen * blockalign; // readlen is the number of *bytes* to read
	chan = ifmthdr.nChannels;

	p = buf + readlen;

	if (bitsamp == 8) cnt = FileRead(inhandle, p,   readlen);  // if 8 bits per sample
	else              cnt = FileRead(inhandle, buf, readlen);

	MainForm->wdatacurr = FileSeek(inhandle, (__int64) 0, 1);  // save current data position

	if(cnt != readlen)
	{
	  if(MainForm->winradwav && !MainForm->wlooping && *iauxihdr.nextfilename != 0)
	  {
		newname = iauxihdr.nextfilename;
		InClose();

		if(!InOpen(iauxihdr.nextfilename))
		{
		  sprintf(s, "File %s could not be opened for reading", newname);
		  warn(s);
		  return;
		}

		MainForm->mustchangecaption = true;
		MainForm->newcaption = ExtractFileName(newname);
		InRewind();
		goto tryagain;
	  }

	  wreadlen = (cnt /= blockalign); // return # of items, not bytes;
	  return;
	}

	if (bitsamp == 8)      // if 8 bits per sample...
	{
	  for (i = 0; i < readlen; i++)
		((short *)buf)[i] = (*p++ - 128) * 256;
	}

	readlen /= blockalign;  // now readlen is the number of samples

	if(format == 1)    // not float format
	  for(k = 0, i = 0; k < readlen;)
	  {
		switch(bitsamp)
		{
		  case 24 :
			threebytes = (tb *)(buf + i); if(chan == 2) i+=3;   // else use same data for right chan
			((int *)ptrL)[k]   = (threebytes->twobytes);        // the 2 LSB
			((int *)ptrL)[k]  |= (threebytes->thirdbyte << 16); // the MSB byte
			if((threebytes->thirdbyte & 0x80) == 0x80)          // expand sign if negative
			  ((int *)ptrL)[k]  |= 0xff000000;

			threebytes = (tb *)(buf + i); i+=3;
			((int *)ptrR)[k]   = (threebytes->twobytes);
			((int *)ptrR)[k]  |= (threebytes->thirdbyte << 16);
			if((threebytes->thirdbyte & 0x80) == 0x80)
			  ((int *)ptrR)[k]  |= 0xff000000;
			k++; break;

		  case 8  :  // bytes have already been converted to short...
		  case 16 :
			((int *)ptrL)[k]   = ((short *)buf)[i]   << 8; if(chan == 2) i++;  // else dup per mono
			((int *)ptrR)[k]   = ((short *)buf)[i++] << 8;
			k++; break;
		}
	  }
	else  // *must* be format 3, float
	{
	  for(k = 0, i = 0; k < readlen;)
	  {
		((int *)ptrL)[k]   = ((float *)buf)[i++] * twoTO23m1;
		((int *)ptrR)[k++] = ((float *)buf)[i++] * twoTO23m1;
	  }

	  if(!MainForm->SwapIQ->Checked) // if float, then suppose it is a FlexRadio Wav file
	  {
		MainForm->SwapIQ->Checked = true;
		MainForm->SwapIQClick(NULL);
	  }
	}

	tmp = MainForm->wdatacurr / (bitsamp * 2 * ifmthdr.nSamplesPerSec / 8);
	secelapsed = tmp - prevsec;
	prevsec = tmp;
	ws = ws + secelapsed;

	while(ws > 59) {ws -= 60; wm++;}
	while(wm > 59) {wm -= 60; wh++;}
	while(wh > 23) {wh -= 24; wd++;}
	k = daymonth[wM]; if(wM == 1) k = (wy == 2008) ? 29 : 28;
	if(wd > k)  {wd = 1; wM++;}
	if(wM > 11) {wM = 0; wy++;}

	MainForm->DisplayWavTime();

	return;
}
//---------------------------------------------------------------------------
void  WavIO::InClose()
{
   if(inhandle > 0) FileClose(inhandle);
   inhandle = -1;
   MainForm->perseuswav = false;
   MainForm->rfspacewav = false;
   MainForm->winradwav  = false;
}
//---------------------------------------------------------------------------
bool WavIO::InOpen(char *name)
{

   int i, c, len;
   char s[256];
   __int64  filelen;

   ircvrhdr.chunkSize = -1;
   strcpy(inname, name);
   inhandle = FileOpen(inname, fmOpenRead);
   if (inhandle < 0)
   {
	  return false;
   }

   FileRead(inhandle, &iriffhdr, sizeof(iriffhdr));
   FileRead(inhandle, &ifmthdr,  sizeof(ifmthdr));
   FileSeek(inhandle, (__int64)(sizeof(iriffhdr) + 8 + ifmthdr.nChunkSize), 0);
   FileRead(inhandle, &idatahdr, sizeof(idatahdr));  // used as 'generic'
													 // header, being the shortest

   while(strnicmp(idatahdr.ckID, "data", 4))
   {
	 if(strnicmp(idatahdr.ckID, "rcvr", 4) == 0)   // then 'rcvr'found
	 {
		MainForm->perseuswav = true;
		len = (int)idatahdr.nChunkSize;    // this really is the length of the rcvr chunk
		FileSeek(inhandle, (int)(0-sizeof(idatahdr)), 1);
		FileRead(inhandle, &ircvrhdr, len + 8);  // it is unsafe to use the sizeof()...
	 }
	 else
	 if(strnicmp(idatahdr.ckID, "auxi", 4) == 0)   // then 'auxi'found, Spectrvue...
	 {
		len = (int)idatahdr.nChunkSize;    // this really is the length of the auxi chunk

		switch(len)
		{
		   case 68      : MainForm->rfspacewav = true; break;
		   case 68 + 96 : MainForm->winradwav  = true; break;
		   default      : goto ignore;    // invalid auxi chunk....
		}

		FileSeek(inhandle, (int)(0-sizeof(idatahdr)), 1);
		FileRead(inhandle, &iauxihdr, len + 8);  // it is unsafe to use the sizeof()...
	 }
	 else
ignore:
		FileSeek(inhandle, (__int64)idatahdr.nChunkSize, 1);

	 FileRead(inhandle, &idatahdr, sizeof(datahdr));
   }

   MainForm->wdatastart  = FileSeek(inhandle, 0, 1);  // save current position (start of data)
   filelen = FileSeek(inhandle, (__int64) 0, 2);
   MainForm->wdatalength = filelen - MainForm->wdatastart; //compute data length
   FileSeek(inhandle, MainForm->wdatastart,0);        // seek again to start of data

   if(strnicmp(iriffhdr.riff, "RIFF", 4) ||
	  strnicmp(iriffhdr.wave, "WAVE", 4) ||
	  strnicmp(ifmthdr.ckID,  "fmt ", 4) ||
	  strnicmp(idatahdr.ckID, "data", 4))
   {
	  sprintf(s, "%s : is not a supported WAV file", name);
	  warn(s);
	  return false;
   }

   bitsamp    =  ifmthdr.nBitsPerSample;
   format     =  ifmthdr.wFormatTag;
   blockalign =  ifmthdr.nBlockAlign;

   if(format != 1 /*uncompressed PCM*/ && format != 3 /* float format*/)
   {
	  sprintf(s, "%s : is not an uncompressed PCM file", name);
	  warn(s);
	  return false;
   }

   nomsamprate = ifmthdr.nSamplesPerSec;

   c = idatahdr.nChunkSize;                 // data length in bytes

   if(bitsamp != 8 && bitsamp != 16 && bitsamp != 24 && bitsamp != 32)
   {
	  sprintf(s, "%s : Unsupported sample format (%d bits per sample)", name, bitsamp);
	  warn(s);
	  return false;
   }

// if the rcvr or auxi chunk was present, set the LO freq.
   if(MainForm->perseuswav || MainForm->rfspacewav || MainForm->winradwav)
   {
	  MainForm->extLOfreq = (ircvrhdr.chunkSize >= 0) ? ircvrhdr.nCenterFrequencyHz
													  : iauxihdr.CenterFreq;

	  MainForm->SetLONixieFreq(MainForm->extLOfreq);
	  for(int k=0; k<NUMDIGITS; k++)  MainForm->ExtLoDigit[k].Show(MainForm->ExtLoDig[k]);

	  MainForm->FF1Scale->StartVal = (MainForm->FF1InizioScala + MainForm->extLOfreq) / 1000.;
	  MainForm->FF1Scale->StopVal  = MainForm->FF1Scale->StartVal + MainForm->FF1span / 1000.;
	  MainForm->FF1Scale->Repaint();
   }

   if(MainForm->perseuswav)
   {
	  tblock = localtime(&ircvrhdr.timeStart);
	  Swy = wy = tblock->tm_year + 1900;
	  SwM = wM = tblock->tm_mon;
	  Swd = wd = tblock->tm_mday;
	  Swh = wh = tblock->tm_hour;
	  Swm = wm = tblock->tm_min;
	  Sws = ws = tblock->tm_sec;
   }
   else
   if(MainForm->rfspacewav || MainForm->winradwav)
   {
	  Swy = wy = iauxihdr.StartTime.wYear;
	  SwM = wM = iauxihdr.StartTime.wMonth - 1;
	  Swd = wd = iauxihdr.StartTime.wDay;
	  Swh = wh = iauxihdr.StartTime.wHour;
	  Swm = wm = iauxihdr.StartTime.wMinute;
	  Sws = ws = iauxihdr.StartTime.wSecond;
   }
   else
	  Swy = SwM = Swd = Swh = Swm = Sws = wy = wM = wd = wh = wm = ws = 0;

   prevsec = 0;

   return true;
}
//---------------------------------------------------------------------------
bool WavIO::OutOpen(char *name)
{
	strcpy(outname, name);
	MainForm->owavsize = 0;
	outhandle = FileCreate(outname);

	if(outhandle > 0)
	{  strncpy(oriffhdr.riff, "RIFF", 4);
	   oriffhdr.filesize = 0L;
	   strncpy(oriffhdr.wave, "WAVE", 4);
	   strncpy(ofmthdr.ckID, "fmt ", 4);
	   strncpy(oauxihdr.chunkID, "auxi", 4);
	   ofmthdr.nChunkSize = 16L;
	   ofmthdr.wFormatTag = 1;
	   ofmthdr.nChannels = 2;
	   ofmthdr.nBitsPerSample  = MainForm->wavbits;
	   ofmthdr.nSamplesPerSec  =  (int) MainForm->srateIn;
	   ofmthdr.nBlockAlign = ofmthdr.nChannels * MainForm->wavbits / 8;
	   ofmthdr.nAvgBytesPerSec = (int) MainForm->srateIn * ofmthdr.nBlockAlign;
	   strncpy(odatahdr.ckID, "data", 4);
	   odatahdr.nChunkSize = 0L;
	   oauxihdr.chunkSize = 9 * sizeof(DWORD) + 2 * sizeof(SYSTEMTIME) +
	                        sizeof(oauxihdr.nextfilename);

	   GetSystemTime(&oauxihdr.StartTime);
	   oauxihdr.CenterFreq = MainForm->extLOfreq;

	   oauxihdr.ADFrequency = 0;  // currently not used
	   oauxihdr.IFFrequency = 0;  // currently not used
	   oauxihdr.Bandwidth   = 0;  // currently not used
	   oauxihdr.IQOffset    = 0;  // currently not used
	   oauxihdr.Unused2 = oauxihdr.Unused3 = oauxihdr.Unused4 = oauxihdr.Unused5 = 0;

	   if(sizeof(oriffhdr) != FileWrite(outhandle, &oriffhdr, sizeof(oriffhdr))) return false;
	   if(sizeof(ofmthdr)  != FileWrite(outhandle, &ofmthdr,  sizeof(ofmthdr)))  return false;
	   if(sizeof(oauxihdr) != FileWrite(outhandle, &oauxihdr, sizeof(oauxihdr))) return false;
	   if(sizeof(odatahdr) != FileWrite(outhandle, &odatahdr, sizeof(odatahdr))) return false;
	   return true;
	}

	return false;
}
//---------------------------------------------------------------------------
bool WavIO::OutWrite(unsigned int * pL, unsigned int * pR, int wlen)
{
	switch(MainForm->wavbits)
	{
	  case 16 :
		for (int k = 0; k < wlen; k++)
		{
		   buf16[k*2]       = pL[k] >> 8;
		   buf16[k*2 + 1]   = pR[k] >> 8;
		}

		return OutWriteBuf(buf16, wlen * 4);  // two samples of two bytes each

	  case 24 :
		for (int k = 0; k < wlen; k++)
		{
			threebytes = &buf24[k*2];
			threebytes->twobytes  = (unsigned short)(pL[k]  & 0xffff);     // the two LSBs
			threebytes->thirdbyte = (unsigned char)((pL[k] >> 16) & 0xff); // the MSB

			threebytes = &buf24[k*2 + 1];
			threebytes->twobytes  = (unsigned short)(pR[k]  & 0xffff);     // the two LSBs
			threebytes->thirdbyte = (unsigned char)((pR[k] >> 16) & 0xff); // the MSB
		}

		return OutWriteBuf(buf24, wlen * 6);  // two samples of three bytes each
	}
	return false;  // not 16 nor 24 ? error...
}
//---------------------------------------------------------------------------
bool WavIO::OutWriteBuf(void *buf, int len)
{
	AnsiString next;

	if(len != FileWrite(outhandle, buf, len)) return false;
	odatahdr.nChunkSize += len;
	oriffhdr.filesize   += len;
	MainForm->owavsize  += len;

	if(oriffhdr.filesize > 2000000000)  // 2 billions of bytes, too near to the limit...
	{
	  next = MainForm->GenerateTimeStamp(".wav");
	  OutClose(next.c_str());
	  if(!OutOpen(next.c_str()))
	  {
		warn("Unable to create a new output WAV file");
		return false;
	  }
	}

	return true;
}
//---------------------------------------------------------------------------
void WavIO::OutClose(char *next)
{
	GetSystemTime(&oauxihdr.StopTime);
	strcpy(oauxihdr.nextfilename, next);

	oriffhdr.filesize += sizeof(oriffhdr) - 8 + sizeof(ofmthdr) + sizeof(oauxihdr) +
						 sizeof(odatahdr);
	FileSeek(outhandle, 4, SEEK_SET);
	FileWrite(outhandle, &oriffhdr.filesize, 4);
	FileSeek(outhandle, (int)(sizeof(oriffhdr) + sizeof(ofmthdr) +
						4 * sizeof(char) + sizeof(long) + sizeof(SYSTEMTIME)), SEEK_SET);
	FileWrite(outhandle, &oauxihdr.StopTime, sizeof(SYSTEMTIME));

	FileSeek(outhandle, (int)(sizeof(oriffhdr) + sizeof(ofmthdr) +
						4 * sizeof(char) + sizeof(long) + 2 * sizeof(SYSTEMTIME) +
						9 * sizeof(DWORD)), SEEK_SET);
	FileWrite(outhandle, &oauxihdr.nextfilename, sizeof(oauxihdr.nextfilename));

    *oauxihdr.nextfilename = 0;  //clear next filename

	FileSeek(outhandle, (int)(sizeof(oriffhdr) + sizeof(ofmthdr) +
							  sizeof(oauxihdr) + 4), SEEK_SET);
	FileWrite(outhandle, &odatahdr.nChunkSize, 4);
	FileClose(outhandle);
}
//---------------------------------------------------------------------------
void WavIO::InRewind(void)
{
	FileSeek(inhandle, MainForm->wdatastart, 0);
	prevsec = 0; ws = Sws; wm = Swm; wh = Swh; wd = Swd; wM = SwM; wy = Swy;
}
//---------------------------------------------------------------------------
void WavIO::InPosition(float pos)
{
	__int64 idx;

	if(pos < 0.F || pos >= 1.F) return;
	idx = (__int64)((MainForm->wdatalength - MainForm->wdatastart) * pos + 0.5);
	idx /= blockalign;
	idx *= blockalign;
	idx += MainForm->wdatastart;

	FileSeek(inhandle, idx, 0);
	prevsec = 0; ws = Sws; wm = Swm; wh = Swh; wd = Swd; wM = SwM; wy = Swy;
}
//---------------------------------------------------------------------------
void WavIO::OutRewind(void)
{
	FileSeek(outhandle, 0, 0);
}
//---------------------------------------------------------------------------
void WavIO::SetSampling(void)
{
	MainForm->OtherRate->Caption = "";
	MainForm->oddRate = false;
	switch(ifmthdr.nSamplesPerSec)
	{
	  case  8000 : MainForm->S8000InClick(NULL);  break;
	  case 11025 : MainForm->S11025InClick(NULL); break;
	  case 16000 : MainForm->S16000InClick(NULL); break;
	  case 22050 : MainForm->S22050InClick(NULL); break;
	  case 24000 : MainForm->S24000InClick(NULL); break;
	  case 32000 : MainForm->S32000InClick(NULL); break;
	  case 44100 : MainForm->S44100InClick(NULL); break;
	  case 48000 : MainForm->S48000InClick(NULL); break;
	  case 88200 : MainForm->S88200InClick(NULL); break;
	  case 96000 : MainForm->S96000InClick(NULL); break;
	  case 176400: MainForm->S176400InClick(NULL);break;
	  case 192000: MainForm->S192000InClick(NULL);break;
	  default    : MainForm->OtherRate->Caption = ifmthdr.nSamplesPerSec;
				   MainForm->SOtherInClick(NULL);
				   MainForm->oddRate = true;      break;
	}
}
//---------------------------------------------------------------------------

