//---------------------------------------------------------------------------
//
// This is the main module of Winrad. It essentially implements the GUI, with its
// TmainForm class, which contains many members (variables) that are used to exhange
// information with the other threads of the program. This module runs in the main
// thread of the Winrad process.
//
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DrLabel"
#pragma link "WFall"
#pragma link "AdBScale"
#pragma link "NakedSlider"
#pragma link "FreeButton"
#pragma link "PhaseMeter"
#pragma link "AdvanceBar"
#pragma link "SMeter"
#pragma link "AdButt"
#pragma link "ModeButt"
#pragma resource "*.dfm"

TMainForm *MainForm;
const ASIOSampleRate asioRates[NUMRATES] =
	  {  8000.,  11025., 16000., 22050., 24000., 32000., 44100.,
		 48000., 88200., 96000., 176400., 192000.
	  };

const char Month[12][4] =
	  {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep",
	   "Oct", "Nov", "Dec"
	  };
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner), movingscale1(false), movingscale2(false), SDR14unplugged(true),
	palettechanged(true), agc_decay(0.9999), ff2Size(FF2SMIN), bpchanged(true),
	WF1resizing(false), WF2resizing(false), mustChoose(false), ff1Size(FF1SMIN),
	bfoChanging(false), wavin(false), paused(false), stopAudio(false), SDR14err(false),
	DenoiserInitialized(false), nr(false), cwp(false), nblanker(false), perseus(false),
	fullview(false), hKernel(NULL), overload(false), extio(false),  submode(both),
	controllingExtHW(false), wlooping(false), zapneeded(false), srchanged(false),
	processPriority(NORMAL_PRIORITY_CLASS), extLOchanged(false), controlKey(false),
	altKey(false), HWM(0), despreading(false), freqerr(0.F), locked(false),
	inSP1(false), inSP2(false), inWF1(false), inWF2(false), perseuswav(false),
	LOlocked(false), audiomute(false), lastX(0), lastY(0), rfspacewav(false),
	PIn(NULL), PFF1(NULL), PFF2(NULL), mustGetTune(false), mustGetMode(false),
	mustGetFilters(false), StopRequested(false), StartRequested(false), wrec(wstopped),
	wavbits(16), werror(false), winradwav(false), mustchangecaption(false)
{
	bIsWindowsXP();
	asioDrivers = new AsioDrivers();
	numASIOdrv = asioDrivers->asioGetNumDev();
	*asioName = 0; asioNum = 0;

	hwnd = Handle;
	windowhandle = MainForm->Handle;
	W = Screen->Width;     H = Screen->Height;
	QueryPerformanceFrequency((LARGE_INTEGER *)&TicksPerSecond);

// disable Windows animation effects
	ai.cbSize = sizeof(ANIMATIONINFO);
	SystemParametersInfo(SPI_GETANIMATION,sizeof(ANIMATIONINFO),&ai,0);
	prevAnim = ai.iMinAnimate;

	if(prevAnim)
	{
	  ai.iMinAnimate = 0;
	  SystemParametersInfo(SPI_SETANIMATION,sizeof(ANIMATIONINFO),&ai,0);
	}

//    SystemParametersInfo(SPI_GETDROPSHADOW, 0, &dropshadow, 0);
//    SystemParametersInfo(SPI_SETDROPSHADOW, 0, FALSE, 0);
	SystemParametersInfo(SPI_GETCOMBOBOXANIMATION, 0, &comboanim, 0);
	SystemParametersInfo(SPI_SETCOMBOBOXANIMATION, 0, FALSE, 0);
	SystemParametersInfo(SPI_GETMENUANIMATION, 0, &menuanim, 0);
	SystemParametersInfo(SPI_SETMENUANIMATION, 0, FALSE, 0);
	SystemParametersInfo(SPI_GETTOOLTIPANIMATION, 0, &toolanim, 0);
	SystemParametersInfo(SPI_SETTOOLTIPANIMATION, 0, FALSE, 0);
//-------------------------


// set the working set sizes min 8 MB and max 64 MB

	SetProcessWorkingSetSize(GetCurrentProcess(),
							 8 * 1024 * 1024, 64 * 1024 * 1024);
//							 QUOTA_LIMITS_HARDWS_ENABLE);

	BAM->Type   = bAM;
	BECSS->Type = bECSS;
	BFM->Type   = bFM;
	BLSB->Type  = bLSB;
	BUSB->Type  = bUSB;
	BCW->Type   = bCW;
	BDRM->Type  = bDRM;

	wfm_rf_cut = 250000;
	wfm_deemph_tau = 5.e-5;
	wfm_audio_cut = 5000;


	//
	// Fill the sin_t and atan_t look-up tables
	//
	int u = 0;
	unsigned short us;
	for(u = 0; u < TRIG_FUNC_TAB_SIZE + TRIG_FUNC_TAB_SIZE / 4; u++)
	{
		sin_t[u] = sin(twoPI * (double(u) + .5)
				/ double(TRIG_FUNC_TAB_SIZE));
		sin_2t[u] = sin(fourPI * (double(u) + .5)
				/ double(TRIG_FUNC_TAB_SIZE));
		sin_3t[u] = sin(sixPI * (double(u) + .5)
				/ double(TRIG_FUNC_TAB_SIZE));
	}
	for(u = 0; u < ATAN_TAB_SIZE; u++)
	{
		atan_t[u] =	atan(16. * (double(u) + .5)
					/ double(ATAN_TAB_SIZE));
	}



	InBufferReady   = CreateEvent(NULL, FALSE, FALSE, NULL);
	InTerminate     = CreateEvent(NULL, FALSE, FALSE, NULL);
	FF1Terminate    = CreateEvent(NULL, FALSE, FALSE, NULL);
	FF2BufferReady  = CreateEvent(NULL, FALSE, FALSE, NULL);
	FF2Terminate    = CreateEvent(NULL, FALSE, FALSE, NULL);

	AddFontResource("tahomabd.ttf");
	SendMessage(HWND_BROADCAST, WM_FONTCHANGE, 0, 0);
	myFont = new TFont;
	myFont->Name = "Tahoma Bold";
	myFont->Size = 10;
	myFont->Color = (TColor)0x006F3731;
	Font = myFont;

	InitializeCriticalSection(&acFF1que);
	InitializeCriticalSection(&acFF2que);
	InitializeCriticalSection(&screenUpdate);
	InitializeCriticalSection(&FDupdate);
	InitializeCriticalSection(&inCallback);
	InitializeCriticalSection(&FifoAccess);

	sav                  = TRect(0, 0,
						   this->Canvas->TextWidth("MNN,MNN.MNN kHz"),
						   2 * this->Canvas->TextHeight("M"));
	wbrush               = new TBrush;
	wbrush->Color        = clSilver;
	whitepen             = new TPen;
	whitepen->Color      = clWhite;
	WF1bm                = new Graphics::TBitmap;
	WF2bm                = new Graphics::TBitmap;
	SP1back              = CreateSolidBrush(RGB(0, 0, 48));
	SP2back              = CreateSolidBrush(RGB(0, 0, 48));
	myBrush              = CreateSolidBrush(RGB(0, 0, 128));
	winback              = CreateSolidBrush(RGB(0, 80, 128));
	redpen               = CreatePen(PS_SOLID, 1, clRed);
	greenpen             = CreatePen(PS_SOLID, 1, clGreen);
	darkpen              = CreatePen(PS_SOLID, 1, RGB(40, 40, 96));
	graypen              = CreatePen(PS_SOLID, 1, RGB(128, 128, 128));
	ypen                 = CreatePen(PS_SOLID, 1, clYellow);
	SP1pen               = CreatePen(PS_SOLID, 1, clSilver);
	SP2pen               = CreatePen(PS_SOLID, 1, clSilver);
	dBcolor              = RGB(128, 128, 160);

	WF1bm->PixelFormat   = pf8bit;
	WF1bm->Palette       = ComputePalette();

	WF2bm->PixelFormat   = pf8bit;
	WF2bm->Palette       = ComputePalette();

	hdcscreen            = GetDC(NULL);
	dBfont               = myCreateFont(hdcscreen, "Arial", 70, 80,
															ATTR_NONE, true);
	CWfont               = myCreateFont(hdcscreen, "Arial", 90, 90,
															ATTR_BOLD, true);

	ComboBox4->Text = String(extLOoffset / 1000);

	ReleaseDC(NULL, hdcscreen);    // no more used

	SP1bottomrange = - 160;       // bottom at -160 dB
	SP1range       = 160;
	SP1toprange    = SP1bottomrange + SP1range;
	SP2bottomrange = - 160;       // bottom at -160 dB
	SP2range       = 160;
	SP2toprange    = SP2bottomrange + SP2range;

	IWav = new WavIO;
	OWav = new WavIO;

// get decimal separator (depending on locale)
	GetLocaleInfo(LOCALE_SYSTEM_DEFAULT, LOCALE_SDECIMAL, sepb, 16);
	sep = sepb[0];

// Intellimouse support

	MainOldProc = MainForm->WindowProc;
	Avg1OldProc = AvgWF1->WindowProc;
	Avg2OldProc = AvgWF2->WindowProc;
	MainForm->WindowProc = MainWheelProc;
	AvgWF1->WindowProc   = Avg1WheelProc;
	AvgWF2->WindowProc   = Avg2WheelProc;

// prepare to support GetSystemTimes
	hKernel = LoadLibrary( _T("Kernel32.dll") );
	if(hKernel != NULL)
	{
	   GSTaddr = (pfnGetSystemTimes)GetProcAddress(hKernel, "GetSystemTimes" );
	   if(GSTaddr == NULL )
	   {
		  FreeLibrary(hKernel);
		  hKernel = NULL;
	   }
	}

	for(int k = 0; k <= SINCLEN; k++)
	{
	  double x = -5.L + (double)k/SINCSTEP;
	  if(k == SINCLEN/2) {sinc[k] = 1.; continue;}
	  sinc[k] = sin(x * PI)/(x * PI);
	}
	nspdWinBlackman(sinc, SINCLEN+1, -0.27);

	int rc = GetProcessAffinityMask(GetCurrentProcess(), &procAffinityMask, &sysAffinityMask);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
	Left = 0;  Top = 0;  Width = W;  Height = H;

	if(W < 1024 || H < 768)
	{
	   fatal("Sorry, Winrad needs a screen resolution of at least 1024x768\n" \
			 "                      (ideally 1280x1024)");
	   Application->Terminate();
	}

	CPUT->Visible    = isXPorGT;
	CPUW->Visible    = isXPorGT;
	LCPUT->Visible   = isXPorGT;
	LCPUW->Visible   = isXPorGT;
	NotAv98->Visible = !isXPorGT;

	bottomtop = H - 320;
	topbottom = bottomtop + 1;

	if(W < 1280  || H < 1024)
	{
	  WF1->Width      = 960;
	  WF1->Height     = 200;
	  FF1Scale->Width = 960;
	  FF1Scale->Top   = WF1->Top + WF1->Height;
	  SP1->Top        = FF1Scale->Top + FF1Scale->Height;
	  SP1->Height     = topbottom - SP1->Top - 20;
	  SP1->Width      = 960;
	  WF2->Width      = 512;
	  WF2->Height     = 110;
	  WF2->Top        = bottomtop + 16;
	  FF2Scale->Top   = WF2->Top + WF2->Height;
	  SP2->Top        = FF2Scale->Top + FF2Scale->Height;
	  SP2->Width      = 512;
	  SP2->Height     = 155;
	  FF2Scale->Width = 512;
	}

	WF1L            = WF1->Left;    // referred to screen
	WF1T            = WF1->Top;     // referred to screen
	WF1W            = WF1->Width;
	WF1H            = WF1->Height;
	WF2L            = WF2->Left;    // referred to screen
	WF2W            = WF2->Width;
	WF2H            = WF2->Height;
	WF2T            = WF2->Top;     // referred to screen

	SP2T            = SP2->Top;     // referred to screen
	SP1L            = SP1->Left;    // referred to screen
	SP1T            = SP1->Top;     // referred to screen
	SP1W            = SP1->Width;
	SP1H            = SP1->Height;
	SP1Base->Left   = WF1L + WF1W + WF1L - SP1Gain->Left - SP1Gain->Width;
	SP2L            = SP2->Left;    // referred to screen
	SP2W            = SP2->Width;
	SP2H            = SP2->Height;

	WF1bm->Height   = WF1H;
	WF1bm->Width    = WF1W;
	WF2bm->Height   = WF2H;
	WF2bm->Width    = WF2W;

	SP1rect         = Rect(0,0,SP1W, SP1H);
	WF1rect         = Rect(0,0,WF1W, WF1H);
	WF1dest         = Rect(0, 1, WF1W, WF1H);
	WF1src          = Rect(0, 0, WF1W, WF1H - 1);
	FF1rect         = Rect(WF1L-1, WF1T-1, WF1L + WF1W + 2,
						   WF1T + WF1H + FF1Scale->Height + SP1H + 1);
	SP2rect         = Rect(0,0,SP2W, SP2H);
	WF2rect         = Rect(0,0,WF2W, WF2H);
	WF2dest         = Rect(0, 1, WF2W, WF2H);
	WF2src          = Rect(0, 0, WF2W, WF2H - 1);

	FF2rect         = Rect(WF2L-1, WF2T-1, WF2L + WF2W + 2,
							   WF2T + WF2H + FF2Scale->Height + SP2H + 1);

	if(W > 1024 && H >= 1024 )
	{

	}
	else              // W == 1024
	{
	  ModePan->Top    = SP1T + SP1H + 1;
	  WPanel->Top     = ModePan->Top + 70;
	  WavName->Top    = WPanel->Top - 36;
	  WavName->Left   = WPanel->Left;
	  WavTime->Left   = WavName->Left;
	  WavTime->Top    = WavName->Top + 14;

	  SP1Gain->Height = topbottom - SP1Gain->Top - 18;
	  SP1Base->Height = topbottom - SP1Base->Top - 18;
	  LPriv->Top      = WPanel->Top + 40;
	  LPriv->Left     = WPanel->Left;
	  Poly1->Top      = LPriv->Top + 16;
	  Poly2->Top      = Poly1->Top;
	  Poly4->Top      = Poly1->Top;
	  Poly1->Left     = LPriv->Left - 4;
	  Poly2->Left     = Poly1->Left + 40;
	  Poly4->Left     = Poly2->Left + 42;
	  Lres->Top       = LPriv->Top + 34;
	  Lres->Left      = LPriv->Left;

	  SP2Gain->Top    = bottomtop + 14;
	  SP2Base->Top    = SP2Gain->Top;
	  AGCGain->Top    = bottomtop;
	  Volume->Top     = bottomtop;
	  LThr->Top       = bottomtop + 137;
	  LVol->Top       = LThr->Top;
	  BMute->Top      = LThr->Top + 16;
	  BMute->Left     = LThr->Left + 3;
	  LAvgWF1->Top    = bottomtop + 280;
	  LAvgWF2->Top    = LAvgWF1->Top;
	  AvgWF1->Top     = LAvgWF1->Top + 18;
	  AvgWF2->Top     = LAvgWF2->Top + 18;
	  LWF2Gain->Top   = bottomtop;
	  LWF2Contrast->Top = bottomtop;
	  WF2Speed->Top   = bottomtop + 304;
	  LWF2Speed->Top  = WF2Speed->Top;
	  WF2Resol->Top   = WF2Speed->Top + 2;
	  L2Resol->Top    = WF2Speed->Top;
	  BFull->Top      = WF2Speed->Top;
	  BWF2Avg->Top    = BFull->Top;
	  CPUT->Top       = BFull->Top;
	  LCPUT->Top      = CPUT->Top - 3;
	  CPUW->Top       = CPUT->Top - 12;
	  LCPUW->Top      = CPUW->Top - 3;
	  LCPULoad->Top   = CPUW->Top - 17;
	  NotAv98->Top    = CPUW->Top;

	  WF1Speed->Top   = FF1rect.bottom;
	  WF1Speed->Max   = 100;
	  WF2Speed->Max   = 20;
	  WF1Speed->Min   = 1;
	  WF2Speed->Min   = 1;
	  WF1Resol->Top   = FF1rect.bottom + 2;
	  L1Resol->Top    = WF1Speed->Top;
	  WF1Resol->Left  = WF2Resol->Left;
	  L1Resol->Left   = WF1Resol->Left + 76;
	  LWF1Speed->Top  = WF1Speed->Top;
	  WF1Slow->Top    = WF1Speed->Top;
	  SP1FSpeed->Top  = WF1Speed->Top;
	  SP1FSpeed->Left = WF1Slow->Left + 80;
	  BWF1Rev->Top    = SP1FSpeed->Top;
	  BWF1Rev->Left   = SP1FSpeed->Left + 36;
	  BWF1Avg->Top    = BWF1Rev->Top;
	  BWF1Avg->Left   = BWF1Rev->Left + 44;

	  WF2Gain->Top    = bottomtop + 2;
	  WF2Contrast->Top= WF2Gain->Top;
	  VU->Top         = bottomtop + 8;
	  AgcFast->Top    = VU->Top + VU->Height - 1;
	  AgcSlow->Top    = AgcFast->Top;
	  LAGC->Top       = AgcFast->Top + 20;
	  Panel1->Left    = SP2Base->Left + SP2Base->Width;
	  Panel1->Top     = Lres->Top + Lres->Height + 15;
	  ModePan->Left   = L1Resol->Left + 112;
	  ModePan->Height = 22;

	  Label2->Top = LWF2Contrast->Top + 5;
	  Label2->Left = LWF2Contrast->Left + LWF2Contrast->Width + 40;
	  Panel2->Top = Poly4->Top;
	  Panel2->Left = Poly4->Left + Poly4->Width + 5;


	  PM->Top         = LAGC->Top + LAGC->Height + 8;

	  Clock->Left     = LCPULoad->Left;
	  Clock->Top      = LCPULoad->Top - 20;

	  RecPanel->Top   = WPanel->Top;
	  RecPanel->Left  = WPanel->Left + WPanel->Width + 1;
	  LRec->Left      = RecPanel->Left;
	  LRec->Top       = RecPanel->Top - 16;
	  LOLock->Left    = LRec->Left;
	  LOLock->Top     = LRec->Top + 39;
	  RecGB->Left     = LOLock->Left;
	  RecGB->Top      = LOLock->Top + 14;

	}

	MainForm->Canvas->Brush = wbrush;
	MainForm->Canvas->Pen   = whitepen;

	WF1->Fill(WF1bm);
	WF2->Fill(WF2bm);

	WF1bm->Canvas->Brush->Style = bsClear;
	WF1bm->Canvas->Brush->Color = (TColor) RGB(0, 0, 48);
	WF1bm->Canvas->FillRect(WF1rect);

	WF2bm->Canvas->Brush->Style = bsClear;
	WF2bm->Canvas->Brush->Color = (TColor) RGB(0, 0, 48);
	WF2bm->Canvas->FillRect(WF2rect);

	hdcscreen = GetDC(NULL);
	SP1bm     = CreateCompatibleBitmap(hdcscreen,
										  SP1W, SP1H + WF1H + FF1Scale->Height);
	SP1bmhdc  = CreateCompatibleDC(hdcscreen);
	SP2bm     = CreateCompatibleBitmap(hdcscreen,
										  SP2W, SP2H + WF2H + FF2Scale->Height);
	SP2bmhdc  = CreateCompatibleDC(hdcscreen);
	ReleaseDC(NULL, hdcscreen);    // no more used

	SelectObject(SP1bmhdc, SP1bm);
	SetBkMode(SP1bmhdc, TRANSPARENT);  // transparent text background
	SelectObject(SP2bmhdc, SP2bm);
	SetBkMode(SP2bmhdc, TRANSPARENT);  // transparent text background

	SelectObject(SP1bmhdc, SP1back);
	SelectObject(SP1bmhdc, dBfont);
	::FillRect(SP1bmhdc, &SP1rect, SP1back);
	SelectObject(SP2bmhdc, SP2back);
	SelectObject(SP2bmhdc, dBfont);
	::FillRect(SP2bmhdc, &SP2rect, SP2back);

	Application->OnMessage = AppMessage;
	Application->OnRestore = (TNotifyEvent)&AppRestore;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
	static bool inited = false;
	float cf;

	if(!inited)
	{
	  inited = true;

// now check for the presence of an extIO DLL and, if found,load the entry points
// addresses

	  extHWtype = none;
	  CoInitialize(NULL);
	  if(GetDLLname(dllname) &&
		 NULL != (hextIO = LoadLibrary( _T(dllname))))
	  {
		 InitHW          = (pfnInitHW) GetProcAddress(hextIO,         "InitHW" );
		 OpenHW          = (pfnOpenHW) GetProcAddress(hextIO,         "OpenHW");
		 CloseHW         = (pfnCloseHW)GetProcAddress(hextIO,         "CloseHW" );
		 StartHW         = (pfnStartHW)GetProcAddress(hextIO,         "StartHW");
		 StopHW          = (pfnStopHW) GetProcAddress(hextIO,         "StopHW");
		 SetCallback     = (pfnSetCallback) GetProcAddress(hextIO,    "SetCallback");
		 SetHWLO         = (pfnSetHWLO)GetProcAddress(hextIO,         "SetHWLO");
		 GetHWLO         = (pfnGetHWLO)GetProcAddress(hextIO,         "GetHWLO");
		 GetStatus       = (pfnGetStatus)GetProcAddress(hextIO,       "GetStatus");
		 GetHWSR         = (pfnGetHWSR) GetProcAddress(hextIO,        "GetHWSR");
		 RawDataReady    = (pfnRawDataReady)GetProcAddress(hextIO,    "RawDataReady");
		 ShowGUI         = (pfnShowGUI)GetProcAddress(hextIO,         "ShowGUI");
		 HideGUI         = (pfnHideGUI)GetProcAddress(hextIO,         "HideGUI");
		 TuneChanged     = (pfnTuneChanged)GetProcAddress(hextIO,     "TuneChanged");
		 GetTune         = (pfnGetTune) GetProcAddress(hextIO,        "GetTune");
		 ModeChanged     = (pfnModeChanged)GetProcAddress(hextIO,     "ModeChanged");
		 GetMode         = (pfnGetMode) GetProcAddress(hextIO,        "GetMode");
		 IFLimitsChanged = (pfnIFLimitsChanged)GetProcAddress(hextIO, "IFLimitsChanged");
		 FiltersChanged  = (pfnFiltersChanged)GetProcAddress(hextIO,  "FiltersChanged");
		 GetFilters      = (pfnGetFilters)GetProcAddress(hextIO,      "GetFilters");

// check that the mandatory entry points are present
		 if(InitHW == NULL  || CloseHW == NULL || OpenHW == NULL ||
			StartHW == NULL || StopHW == NULL  || SetCallback == NULL ||
			SetHWLO == NULL || GetStatus == NULL)
		 {
			FreeLibrary(hextIO);
			hextIO = NULL;
			SelExtHW->Visible = false;
		 }
		 else
		 {
			SelExtHW->Visible = true;
			extHWready = InitHW(extHWname, extHWmodel, (int &)extHWtype);
			SetCallback(extIOCallback);
			SelExtHW->Caption = extHWname;
		 }
	  }
	  else
		 SelExtHW->Visible = false;

// prepare the Nixie digits display for the internal NCO...

	  int LODigStart = WF1rect.Right - 20 * 7 - 4;// + 4;

	  for(int k=0; k<NUMDIGITS; k++)
	  {
		inNCODig[k] = false;
		NCODig[k] = new TImage(this);
		NCODig[k]->Parent = this;
		NCODig[k]->Width = 20;
		NCODig[k]->Height = 24;
		NCODig[k]->Top = 30;
		NCODig[k]->Left = 510 + k * 20 +  (k > 2 ? 10 : 0) +  // 1st dec.point at 570
										  (k > 5 ? 10 : 0);   // 2nd dec.point at 640
		NCODig[k]->Visible = true;
		NCODig[k]->Transparent = false;
		NCODig[k]->BringToFront();
		NCODig[k]->Repaint();
		NCODig[k]->OnMouseDown = DigitMouseDown;
		NCODig[k]->OnMouseUp   = DigitMouseUp;
		NCODig[k]->Cursor = crHandPoint;
	  }
	  NCODecPt1->Left = 570;   NCODecPt1->Top = 43;
	  NCODecPt2->Left = 640;   NCODecPt2->Top = 43;

	  LNCO->Left = NCODig[8]->Left + 26;
	  LNCO->Top  = NCODig[8]->Top + 4;

	  NCODigit = new TNCODigits[NUMDIGITS];
	  for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Init(k);

// prepare the Nixie digits display for the external HW LO...

	  for(int k=0; k<NUMDIGITS; k++)
	  {
		inExtLoDig[k] = false;
		ExtLoDig[k] = new TImage(this);
		ExtLoDig[k]->Parent = this;
		ExtLoDig[k]->Width = 20;
		ExtLoDig[k]->Height = 24;
		ExtLoDig[k]->Top = 30;
		ExtLoDig[k]->Left = LODigStart + k * 20 + (k > 2 ? 10 : 0) + // dec.point1 at LODigStart + 40
											(k > 5 ? 10 : 0);  // dec.point2 at LODigStart + 110
		ExtLoDig[k]->Visible = true;
		ExtLoDig[k]->Transparent = false;
		ExtLoDig[k]->BringToFront();
		ExtLoDig[k]->Repaint();
		ExtLoDig[k]->OnMouseDown = DigitMouseDown;
		ExtLoDig[k]->OnMouseUp   = DigitMouseUp;
		ExtLoDig[k]->Cursor = crHandPoint;
	  }
	  LODecPt1->Left = LODigStart + 40;;  LODecPt1->Top = 43;
	  LODecPt2->Left = LODigStart + 110;  LODecPt2->Top = 43;

	  LExtLo->Left = ExtLoDig[0]->Left - LNCO->Width;// - 8;
	  LExtLo->Top  = NCODig[0]->Top + 4;

	  ExtLoDigit = new TExtLoDigits[NUMDIGITS];
	  for(int k=0; k<NUMDIGITS; k++)  ExtLoDigit[k].Init(k);

	  LoadSettings("LastUsed");
	  ApplySettings(Sender);

	  InitLO();
	  SetParam(Sender);
	  SP1MaxVal = ff1Size/2;
	  SP2MaxVal = ff2Size/2;
	  DoWF1(NULL);

	  NBlanker->ShowHint = true;
	  BZAP->ShowHint     = true;
	  NRed->ShowHint     = true;
	  CWPk->ShowHint     = true;
	  BUSB->ShowHint     = true;
	  BLSB->ShowHint     = true;
	  BCW->ShowHint      = true;
	  BAFC->ShowHint     = true;

	  Application->OnIdle = MyIdleHandler;
	}
}
//---------------------------------------------------------------------------
// This routine sets the main parameters after major changes done by the user
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetParam(TObject* Sender)
{
	float srateI, srateO, cf1, maxDispIn, maxDispOut;

	oddRate = false;

	S8000In->Down  = false;   S11025In->Down = false;   S16000In->Down = false;
	S22050In->Down = false;   S24000In->Down = false;   S32000In->Down = false;
	S44100In->Down = false;   S48000In->Down = false;   S88200In->Down = false;
	S96000In->Down = false;   S176400In->Down = false;  S192000In->Down = false;
	SOtherIn->Down = false;

	switch(srateIn)
	{
	  case  8000 :  vsrOut = 8000;    S8000In->Down   = true; break;
	  case 16000 :  vsrOut = 11025;   S16000In->Down  = true; break;
	  case 24000 :  vsrOut = 11025;   S24000In->Down  = true; break;
	  case 32000 :  vsrOut = 11025;   S32000In->Down  = true; break;
	  case 48000 :  vsrOut = 11025;   S48000In->Down  = true; break;
	  case 96000 :  vsrOut = 11025;   S96000In->Down  = true; break;
	  case 192000:  vsrOut = 11025;   S192000In->Down = true; break;

	  case 11025 :  vsrOut = 11025;  S11025In->Down  = true; break;
	  case 22050 :  vsrOut = 11025;  S22050In->Down  = true; break;
	  case 44100 :  vsrOut = 11025;  S44100In->Down  = true; break;
	  case 88200 :  vsrOut = 11025;  S88200In->Down  = true; break;
	  case 176400:  vsrOut = 11025;  S176400In->Down = true; break;

	  default    :  oddRate = true;  SOtherIn->Down  = true;
					if(srateIn >= 11025) vsrOut = 11025; else vsrOut = 8000;
					break;
	}

	if(driver == ASIO) // in ASIO mode the upsampler needs an integer ratio
	{                  // between internal and output sampling rates
	  switch(srateIn)
	  {
		 case 16000 :  vsrOut = 8000; S16000In->Down  = true; break;
		 case 24000 :  vsrOut = 8000; S24000In->Down  = true; break;
		 case 32000 :  vsrOut = 8000; S32000In->Down  = true; break;
		 case 48000 :  vsrOut = 8000; S48000In->Down  = true; break;
		 case 96000 :  vsrOut = 8000; S96000In->Down  = true; break;
		 case 192000:  vsrOut = 8000; S192000In->Down = true; break;
	  }
	}

	if(mode == DRM || mode == FM)
	{
	  // vsrOut = 24000;
	  vsrOut = 48000;
	  bpHicut = 15000;

	  if(srateIn <  24000)
	  {
		srateIn = 24000;
		S24000In->Down  = true;
		S8000In->Down   = false;
		S11025In->Down  = false;
		S16000In->Down  = false;
		S22050In->Down  = false;
	  }

	  if(driver == ASIO)
	  {
		if(srateIn == 32000)  { srateIn = 48000;  S32000In->Down = false;
								S48000In->Down  = true; }

		if(srateIn == 44100)  { srateIn = 48000;  S44100In->Down = false;
								S48000In->Down  = true; }

		if(srateIn == 88200)  { srateIn = 96000;  S88200In->Down = false;
								S96000In->Down  = true; }

		if(srateIn == 176400) { srateIn = 192000; S176400In->Down = false;
		                        S192000In->Down = true; }
	  }
	}

	downFactor = (float)srateIn / vsrOut;
	RatePanel->Height = oddRate ? 347 : 310;
	BClose->Top       = oddRate ? 313 : 289;

	if(driver == ASIO)
	  srateOut = srateIn;
	else
	  srateOut = vsrOut;

	switch(vsrOut)
	{
	  case  8000 :  S8000OutClick(Sender);  break;
	  case 11025 :  S11025OutClick(Sender); break;
	  case 24000 :  S24000OutClick(Sender); break;
	  case 48000 :  S48000OutClick(Sender); break;
	}

	bpchanged = true;
	cf1 = FF1freq;

	srateI = srateIn;
	srateO = vsrOut;

	hibound = srateI / 2.;
	lobound = -hibound;

	maxDispIn  = srateI / 2;
	maxDispOut = srateO / 2.;
	FF1HzPerPixel = srateI / ff1Size;
	FF1span = FF1HzPerPixel * WF1->Width;

	if(datamode == mIQ)
	   FF1InizioScala  = max(-maxDispIn, cf1 - FF1span / 2);
	else
	   FF1InizioScala  = max(0.F, cf1 - FF1span / 2);

	FF1InizioScala     = min(FF1InizioScala, maxDispIn - FF1span);
	FF1Scale->StartVal = (FF1InizioScala + extLOfreq + extLOoffset) / 1000.;
	FF1Scale->StopVal  = (FF1InizioScala + extLOfreq + extLOoffset + FF1span) / 1000.;

	if(controllingExtHW && IFLimitsChanged != NULL)
	  IFLimitsChanged(FF1InizioScala + extLOfreq, FF1InizioScala + extLOfreq + FF1span);

	FF2HzPerPixel      = srateO / ff2Size;
	FF2span            = FF2HzPerPixel * WF2->Width;
	FF2InizioScala     = max(0.F, FF2cf - FF2span / 2);
	FF2Scale->StartVal = FF2InizioScala;
	FF2Scale->StopVal  = FF2InizioScala + FF2span;

	if(FF1Scale->StopVal > (maxDispIn + extLOfreq + extLOoffset) / 1000.)
	{
	   FF1Scale->StopVal  = (maxDispIn + extLOfreq + extLOoffset) / 1000.;
	   FF1Scale->StartVal = FF1Scale->StopVal - (FF1span / 1000.);
	   FF1InizioScala     = FF1Scale->StartVal * 1000. - extLOfreq - extLOoffset;
	   if(controllingExtHW && IFLimitsChanged != NULL)
		 IFLimitsChanged(FF1InizioScala + extLOfreq + extLOoffset, FF1InizioScala + extLOfreq + extLOoffset + FF1span);
	}
	if(FF2Scale->StopVal > maxDispOut)
	{
	   FF2Scale->StopVal  = maxDispOut;
	   FF2Scale->StartVal = FF2InizioScala = FF2Scale->StopVal - FF2span;
	}

	SP1MaxVal          = ff1Size/2;
	SP2MaxVal          = ff2Size/2;

	if(FF1freq + extLOfreq + extLOoffset > FF1Scale->StopVal * 1000. ||
	   FF1freq + extLOfreq + extLOoffset < FF1Scale->StartVal * 1000.)
	{
	  FF1freq = 0;
	}

	SetLONixieFreq((int)(extLOfreq + 0.5));
	for(int k=0; k<NUMDIGITS; k++)  ExtLoDigit[k].Show(ExtLoDig[k]);
	for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);

	AdjustBlueWindow();

	rxband.top    = 0;
	rxband.bottom = FF1rect.Height();

	sprintf(s, "RBW %4.1f Hz", FF1HzPerPixel);
	L1Resol->Caption = s;
	sprintf(s, "RBW %4.1f Hz", FF2HzPerPixel);
	L2Resol->Caption = s;

	FF2cf = (FF2locut + FF2hicut) / 2.;
}
//---------------------------------------------------------------------------
// This routine manages the pressing of the Start/Stop button, starting and stopping
// threads, allocating and deallocating memory and the sound card (if used)
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartStopClick(TObject *Sender)
{
	if(running)
	{
	  running = false;
	  ::SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
	  ::SetThreadPriority((HANDLE) PIn->Handle,  THREAD_PRIORITY_NORMAL);

	  SetEvent(InTerminate);   PIn->WaitFor();
	  SetEvent(FF1Terminate);  PFF1->WaitFor();
	  SetEvent(FF2Terminate);  PFF2->WaitFor();

	  if(controllingExtHW)
	  {
		Sleep(1000);
		switch(extHWtype)
		{
		  case SDR14     :
		  case USBdata16 :
		  case USBdata24 :
		  case USBdata32 :
		  case USBfloat32 :
				 StopHW();
//                 delete[] extIOdata;
				 delete   USBfifo;
				 delete   USBfifo24;  // delete accepts a NULL pointer...
				 delete   USBfifo32;  // delete accepts a NULL pointer...
				 break;

		  case SDRX    :
		  case SCdata  :
				 StopHW();
				 break;
		}
	  }

	  if(driver == WMME)
	  {
		if(!wavin && !extio) SoundCloseIn();
		SoundCloseOut();
	  }
	  else
	  { StopASIO();

// Free the various ring buffers used by Winrad
		delete ILring;
		delete IRring;
		delete OLring;
		delete ORring;
		delete ASIOConv;
	  }

	  delete PIn;   PIn   = NULL;
	  delete PFF1;  PFF1  = NULL;
	  delete PFF2;  PFF2  = NULL;
	  delete Decring;
	  delete PSring;
	  delete Dn;

	  StartStop->DrawColor = clButtOff;
	  StartStop->Font->Color = clTextOff;
	  StartStop->Caption = "Start";
	  SP1Gain->Visible = false;
	  SP1Base->Visible = false;
	  SP2Gain->Visible = false;
	  SP2Base->Visible = false;
	  RecPanel->Visible = false;
	  VU->Position     = 0;
	}
	else
	{
	  justStarted = true;
	  restarted = false;
	  DenoiserInitialized = false;
	  PFF1  = new TProcessFF1(false);  // thread created not suspended
	  PFF2  = new TProcessFF2(false);  // thread created not suspended
	  PIn   = new TProcessIn(false);   // thread created not suspended

	  FiltersInit();

// Allocate some ring buffers
	  Decring  = new TCRing(RINGSIZE);
	  PSring   = new TCRing(RINGSIZE);
	  Dn       = new TDnSample;

	  ::SetPriorityClass(GetCurrentProcess(), processPriority);
	  ::SetThreadPriority((HANDLE) PIn->Handle,  THREAD_PRIORITY_ABOVE_NORMAL);
	  //::SetThreadPriority((HANDLE) PIn->Handle,  THREAD_PRIORITY_HIGHEST);

	  if(driver == WMME)
	  {
// the order of OpenIn, waveInStart and OpenOut seems to be important....
		if(!wavin && !extio && !SoundOpenIn())
		   goto stop_threads;
		if(!wavin && !extio) waveInStart(hin);   // start input loop
		if(!SoundOpenOut()) goto stop_threads;
	  }
	  else   // ASIO drivers
	  {
		if(!InitializeASIO(false))  // false means NOT dummy, but real init
		  goto stop_threads;        // (in other words, PIn has been instantiated already...)

// Allocate some other ring buffers
		ILring = new TVRing(RINGSIZE, asioSampleLength);
		IRring = new TVRing(RINGSIZE, asioSampleLength);
		OLring = new TVRing(RINGSIZE, asioSampleLength);
		ORring = new TVRing(RINGSIZE, asioSampleLength);
		ASIOConv = new ASIOConvertSamples;

		if(!RunASIO())
		  goto stop_threads;
	  }

	  if(controllingExtHW)
	  {
		switch(extHWtype)
		{
		  case SDR14     :
		  case USBdata16 :
			   wavbits = 16;
			   if((numextIQpairs = StartHW(extLOfreq)) <= 0)
			   {
				 goto stop_threads;
			   }
			   USBfifo = new TIQFifo(512, numextIQpairs);
			   // 512 entries of numextIQpairs IQ pairs each

			   inQued = 0;
			   break;

		  case USBdata24 :
		  case USBdata32 :
		  case USBfloat32 :
			   wavbits = 24;    // record 24 bits also for 32 bit floats
			   if((numextIQpairs = StartHW(extLOfreq)) <= 0)
			   {
				 goto stop_threads;
			   }
			   // 512 entries of numextIQpairs IQ pairs each
			   if(extHWtype == USBdata24)
				  USBfifo24 = new TIQFifo24(512, numextIQpairs);
			   else
				  USBfifo32 = new TIQFifo32(512, numextIQpairs);

			   inQued = 0;
			   break;

		  case SDRX    :
		  case SCdata  :
			   wavbits = 16;
			   StartHW(extLOfreq);
			   break;
		}
	  }

	  HWM = 0;
	  running = true;
	  StartStop->DrawColor = clButtOn;
	  StartStop->Font->Color = clTextOn;
	  StartStop->Caption = "Stop";
	  SP1Gain->Visible = true;
	  SP1Base->Visible = true;
	  SP2Gain->Visible = true;
	  SP2Base->Visible = true;
	  RecPanel->Visible = true;  RecPanel->Width = 61;

	  if(wavin)
	  {
		if(driver == WMME)
		{ PIn->wreadlen = BUFSIZE;
		  IWav->InRead(PIn->inL, PIn->inR, PIn->wreadlen);
		  inQued = 1;
		}
		SetEvent(InBufferReady);
	  }
	}

	return;

stop_threads:
	::SetThreadPriority((HANDLE) PIn->Handle,  THREAD_PRIORITY_NORMAL);

	SetEvent(InTerminate);   PIn->WaitFor();
	SetEvent(FF1Terminate);  PFF1->WaitFor();
	SetEvent(FF2Terminate);  PFF2->WaitFor();
}
//---------------------------------------------------------------------------
// This routine gets executed when nothing else is ready for execution. Do here
// some ancilalry tasks, as managing the mouse cursor
//---------------------------------------------------------------------------
void __fastcall TMainForm::MyIdleHandler(TObject *Sender, bool &Done)
{
	static bool firstTime = true, oldover = false;
	static int  oldfreq = 0, oldX = -1, oldY = -1;
	char msg[80];
	bool prev;

	Done = true;

	if(srchanged)
	{
	  srchanged = false;
	  bool wasrunning = running;
	  if(running) StartStopClick(Sender);

	  srateIn = GetHWSR();

	  OtherRate->Caption = srateIn;
	  SOtherInClick(Sender);
	  oddRate = true;

	  SetParam(Sender);
	  InRates->Enabled = false;
	  if(wasrunning) StartStopClick(Sender);
	}

	if(SDR14err)
	{
	  SDR14err = false;
	  StartStopClick(Sender);   // stop everything
	  Sleep(1000);
	  StartStopClick(Sender);   // and restart again
	}

	if(running && extio && extHWtype == SDR14 && SDR14unplugged)
	  StartStopClick(Sender);

	if(firstTime)
	{
	   firstTime = false;
	   GetCurrentCard();

	   Den2AdjForm->Den2QFac->Position =
								 (int)((Den2qfac - 0.95) / 0.049 * 1000. + 0.5);
	   BlkAdjForm->BlkThreshold->Position =
					  (int)((pow(10.F, 1. - BlkThr/10.) - 1.) * 1000./9.) + 0.5;

// the focus is perhaps on a hidden window for ecternal I/O, so the mouse wheel
// does not respond unless we set back the focus to the main window
	   SetFocus();
	}

	if(overload && !vollocked)
		 Volume->Position = Volume->Position * 0.99 + 0.5 ;

	if(overload != oldover)
	{
	  if(overload)
		Volume->FocusColor = clRed;
	  else
		Volume->FocusColor = clGray;

	  Volume->Repaint();
	  oldover = overload;
	}

	GetCursorPos(&mousePos);
	if(mousePos.x == oldX && mousePos.y == oldY) goto mouse_end;
	oldX = mousePos.x;
	oldY = mousePos.y;

// check if in LO digits
	if(mousePos.y < ExtLoDig[0]->Top || mousePos.y > (ExtLoDig[0]->Top + ExtLoDig[0]->Height))
	{ for(int k = 0; k < NUMDIGITS; k++)
		 if(inExtLoDig[k])
		 { inExtLoDig[k] = false;
		   ExtLoDigit[k].bright = false;
		   ExtLoDigit[k].Show(ExtLoDig[k]);
		 }
		 inLO = false;
	}
	else
	{ inLO = false;
	  for(int k = 0; k < NUMDIGITS; k++)
	  { inExtLoDig[k] = mousePos.x > ExtLoDig[k]->Left &&
					 mousePos.x < (ExtLoDig[k]->Left + ExtLoDig[k]->Width);

		prev = ExtLoDigit[k].bright;

		if(inExtLoDig[k])
		{
		  ExtLoDigit[k].bright = inLO = true;
		  MainForm->SetFocus();
		}
		else
		  ExtLoDigit[k].bright = false;

		if(prev ^ ExtLoDigit[k].bright) ExtLoDigit[k].Show(ExtLoDig[k]);
	  }
	}

// check if in NCO digits
	if(mousePos.y < NCODig[0]->Top || mousePos.y > (NCODig[0]->Top + NCODig[0]->Height))
	{ for(int k = 0; k < NUMDIGITS; k++)
		 if(inNCODig[k])
		 { inNCODig[k] = false;
		   NCODigit[k].bright = false;
		   NCODigit[k].Show(NCODig[k]);
		 }
		 inNCO = false;
	}
	else
	{ inNCO = false;
	  for(int k = 0; k < NUMDIGITS; k++)
	  { inNCODig[k] = mousePos.x > NCODig[k]->Left &&
					 mousePos.x < (NCODig[k]->Left + NCODig[k]->Width);

		prev = NCODigit[k].bright;

		if(inNCODig[k])
		{
		  NCODigit[k].bright = inNCO = true;
		  MainForm->SetFocus();
		}
		else
		  NCODigit[k].bright = false;

		if(prev ^ NCODigit[k].bright) NCODigit[k].Show(NCODig[k]);
	  }
	}

// See if the mouse cursor is in the upper or lower waterfall/spectrum
	mouseSP1.x = mouseSP2.x = mousePos.x;
	mouseSP1.y = mouseSP2.y = mousePos.y;
	mouseWF1.x = mouseWF2.x = mousePos.x;
	mouseWF1.y = mouseWF2.y = mousePos.y;

	mouseSP1.x -= SP1L; mouseSP1.y -= SP1T;
	mouseSP2.x -= SP2L; mouseSP2.y -= SP2T;
	mouseWF1.x -= WF1L; mouseWF1.y -= WF1T;
	mouseWF2.x -= WF2L; mouseWF2.y -= WF2T;

	inSP1 = !(mouseSP1.x < 0 || mouseSP1.x >= SP1W ||
			  mouseSP1.y < 0 || mouseSP1.y >= SP1H);
	inSP2 = !(mouseSP2.x < 0 || mouseSP2.x >= SP2W ||
			  mouseSP2.y < 0 || mouseSP2.y >= SP2H);
	inWF1 = !(mouseWF1.x < 0 || mouseWF1.x >= WF1W ||
			  mouseWF1.y < 0 || mouseWF1.y >= WF1H);
	inWF2 = !(mouseWF2.x < 0 || mouseWF2.x >= WF2W ||
			  mouseWF2.y < 0 || mouseWF2.y >= WF2H);


	if(inSP2)
	{
	  float mousefreq = X2F2(mouseSP2.x);

	  if(mousefreq >= bpLocut - 40 && mousefreq <= bpLocut + 40)
	  { Lnear = true;
		Cursor = crSizeWE;
	  }
	  else
	  { Lnear = false;
		Cursor = crCross;
	  }

	  if(!Lnear)
		if(mousefreq >= bpHicut - 40 && mousefreq <= bpHicut + 40)
		{ Hnear = true;
		  Cursor = crSizeWE;
		}
		else
		{ Hnear = false;
		  Cursor = crCross;
		}
	}


	if(inSP1)  // if the mouse cursor is in the upper spectrum
	{
	  float mousefreq = X2F1(mouseSP1.x), low, high;
	  switch(mode)
	  {
		  case FM   :
		  case ECSS :
		  case AM   : low = FF1freq - FF2hicut;  high = FF1freq + FF2hicut; break;
		  case CW   : low = FF1freq - CWPitch + FF2locut;
					  high = FF1freq + FF2hicut - CWPitch; break;
		  case DRM  :
		  case USB  : low = FF1freq + FF2locut;  high = FF1freq + FF2hicut; break;
		  case LSB  : low = FF1freq - FF2hicut;  high = FF1freq - FF2locut; break;
	  }

	  if(mousefreq >= low && mousefreq <= high)
	  { inBetween = true;
		Cursor = crDrag;
	  }
	  else
	  { inBetween = false;
		Cursor = crCross;
	  }

	  if(!running && SP1drag) DoWF1(NULL); // moves the blue window even when not running
	}
	else
	{ if(!inSP2)
		Cursor = crCross;
	  inBetween = false;
	}


// Mouse tied announcements -----------------------

	if(inSP1)
	{
	  Clx = mouseSP1.x + 8;  Cly = mouseSP1.y - 2 * Canvas->TextHeight("M");

	  Cly = max(0, Cly);
	  Cly = min(SP1H - sav.Height(), Cly);

	  EnterCriticalSection(&screenUpdate);
		DynWrite();
	  LeaveCriticalSection(&screenUpdate);

	  lastY = Cly;
	}



// end of mouse position checking

mouse_end:

	if(palettechanged)
	{
	  WF1bm->Palette       = ComputePalette();
	  WF2bm->Palette       = ComputePalette();
	  palettechanged = false;
	}

	int newfreq = (int)(FF1freq + extLOfreq + 0.5);
	if(oldfreq != newfreq)
	{
	  oldfreq = newfreq;
	  SetNCONixieFreq(newfreq);
	  for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
	}

	if(driver == mustStop)
	{
	  driver = driversave;
	  QuitButtClick(Sender);
	}

// if reading from WAV file, update progress display
	if(running && wavin && wdatalength > 0)
	{
	  WTrack->Position = ((double)wdatacurr / (double)wdatalength) * 100. + 0.5;
	}

	if(zapneeded)  // this looks starnge, but it is just a place-holder for something
	{              // I intended to do, but never did
	  zapneeded = false;
	}

	if(extLOchanged)
	{
	  extLOchanged = false;
	  extLOfreq = GetHWLO();
	  SetLONixieFreq(extLOfreq);
	  for(int k=0; k<NUMDIGITS; k++)  ExtLoDigit[k].Show(ExtLoDig[k]);

	  FF1Scale->StartVal = (FF1InizioScala + extLOfreq + extLOoffset) / 1000.;
	  FF1Scale->StopVal  = FF1Scale->StartVal + FF1span / 1000.;
	  FF1Scale->Repaint();

	  if(controllingExtHW && IFLimitsChanged != NULL)
		IFLimitsChanged(FF1InizioScala + extLOfreq, FF1InizioScala + extLOfreq + FF1span);
	}

	if(isXPorGT)
	  if(!GetProcessMemoryInfo(GetCurrentProcess(), &pmc, sizeof(pmc)))
		 ZeroMemory((char *)&pmc, sizeof(pmc));
	  else;
	else
	  ZeroMemory((char *)&pmc, sizeof(pmc));

	if(FStatus->Visible)
	{
	  FStatus->InQue->Caption    = (driver == ASIO) ? "N/A (Asio)" : itoa(inQued, msg, 10);
	  FStatus->OutQue->Caption   = (driver == ASIO) ? "N/A (Asio)" : itoa(normQue, msg, 10);
	  sprintf(msg, "%7.3f", resampfac);
	  FStatus->DownFact->Caption = msg;
	  sprintf(msg, "%5.1f", 1000. * asioDriverInfo.inputLatency / (float)srateIn);
	  FStatus->InLat->Caption  = (driver == ASIO) ? msg : "  N/A (MME)";
	  sprintf(msg, "%5.1f", 1000. * asioDriverInfo.outputLatency / (float)srateIn);
	  FStatus->OutLat->Caption = (driver == ASIO) ? msg : "  N/A (MME)";
	  float f = pmc.WorkingSetSize / 1024.;
	  FStatus->curWSsize->Caption = FormatFloat("###,###   kB", f);
	  sprintf(msg, "%d", pmc.PageFaultCount);
	  FStatus->pageFault->Caption = msg;
	}

	if(mode == ECSS && locked == true)
	  PM->OKZoneColor = clGreen;
	else
	  PM->OKZoneColor = clNavy;

	if(mustGetMode)  // all the mustGetxxxx are set by extIO callback
	{
	  if(GetMode != NULL)
		switch(char c = GetMode())
		{
		  case 'a' :
		  case 'A' : BAMClick(Sender); break;

		  case 'e' :
		  case 'E' : BECSSClick(Sender); break;

		  case 'f' :
		  case 'F' : BFMClick(Sender); break;

		  case 'l' :
		  case 'L' : BLSBClick(Sender); break;

		  case 'u' :
		  case 'U' : BUSBClick(Sender); break;

		  case 'c' :
		  case 'C' : BCWClick(Sender); break;

		  case 'd' :
		  case 'D' : BDRMClick(Sender); break;
		}

	  mustGetMode = false;
	}

	if(mustGetFilters)
	{
	  int lo, hi, pitch;

	  if(GetFilters != NULL)
	  {
		GetFilters(lo, hi, pitch);
		FF2locut = (float) lo;
		FF2hicut = (float) hi;
		bpLocut = FF2locut = min(FF2cf - 5., FF2locut);
		bpHicut = FF2hicut = max(FF2cf + 5., FF2hicut);

		if(mode == CW)
		{
		  bpHicut = min(FF2hicut, vsrOut/2. - 50.F);
		  bpLocut = max(FF2locut, 50.F);
		  bw = bpHicut - bpLocut;

		  if(pitch >= 100 && pitch <= 3000)
		  {
			CWPitch = (float) pitch;
			FF2cf    = CWPitch;
			bpLocut  = FF2locut = FF2cf - bw/2;
			bpHicut  = FF2hicut = FF2cf + bw/2;
			SetParam(Sender);
		  }
		}
		else if(mode != DRM)
		{
		  SSBFF2locut = FF2locut;
		  SSBFF2hicut = FF2hicut;
		}

		bpchanged = true;
		AdjustBlueWindow();
		FF2cf = (FF2locut + FF2hicut) / 2.;

	  }
	  mustGetFilters = false;
	}

	if(mustGetTune)
	{
	   long freq;
	   float Tfreq;

	   if(GetTune != NULL)
	   {
		 freq = GetTune();

		 try
		 {
		   Tfreq = freq;;
		 }
		 catch(...)
		 {
		   goto badfreq;
		 }

		 if((Tfreq - extLOfreq) < lobound || (Tfreq - extLOfreq) > hibound)
		 {
		   goto badfreq;
		 }

		 FF1freq = Tfreq - extLOfreq;
		 if(GetNCONixieFreq() != Tfreq)
		 {
		   SetNCONixieFreq(Tfreq);
		   for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
		 }
		 AdjustBlueWindow();

badfreq:
	   }
	   mustGetTune = false;
	}

	if(StopRequested && running)     // obey callback code 108
	   StartStopClick(Sender);

	StopRequested = false;

	if(StartRequested && !running)   // obey callback code 107
	   StartStopClick(Sender);

	StartRequested = false;

	if(werror)
	{
	   warn("Error writing output WAV file. Recording aborted");
	   WRecClick(Sender);  // will force the stop of the recording
	   werror = false;     // must at the end of the if group
	}

	if(mustchangecaption)
	{
	   mustchangecaption = false;
	   WavName->Caption = newcaption;
	}

	if(wrec == wrecording)
	{
	   static int oldsize = 0;

	   if(oldsize != owavsize)
	   {
		 oldsize = owavsize;
         DisplayRecTime();
	   }
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DynWrite(void)
{
	SP1Restore(lastX, lastY);
	SP1Write(Clx, Cly);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1Restore(int x, int y)
{
	hdcscreen = GetDC(hwnd);
	BitBlt(hdcscreen, x + SP1L, SP1T + y, sav.Width(), sav.Height(),
		   MainForm->SP1bmhdc, x, y, SRCCOPY);
	ReleaseDC(MainForm->hwnd, hdcscreen);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1Write(int x, int y)
{
	char str[24];
	int freq, iM, ik, iH;

	hdcscreen = GetDC(hwnd);
	SetBkMode(hdcscreen, TRANSPARENT);
	SetTextColor(hdcscreen, clYellow);

	freq = (int)(X2F1(mouseSP1.x) + extLOfreq + 0.5);

	if(abs(freq) <= 999999)
	{
	   ik = freq / 1000;
	   iH = freq - ik * 1000;
	   iH = abs(iH);
	   if(freq < 0 && ik == 0)
		 sprintf(str, "-%d,%03d Hz", ik, iH);
	   else
		 sprintf(str, "%d,%03d Hz", ik, iH);
	}
	else
	{
	   iM = freq / 1000000;
	   ik = (freq - iM * 1000000) / 1000;
	   iH = freq - iM * 1000000 - ik * 1000;
	   sprintf(str, "%d,%03d.%03d kHz", iM, ik, iH);
	}

	sav.Right = Canvas->TextWidth(str) + 10;
	x = min(SP1W - sav.Width(), x);
	lastX = x;

	TextOut(hdcscreen, x + SP1L, y + SP1T + 15, str, strlen(str));

	sprintf(str, "%6.1f dB", getdBfromy(mouseSP1.y));
	TextOut(hdcscreen, x + SP1L, y + SP1T, str, strlen(str));
	ReleaseDC(MainForm->hwnd, hdcscreen);

	return;
}
//---------------------------------------------------------------------------
// This routine applies all the saved settings from the previous exeecution
// which were saved into the registry, and read from there by LoadSettings()
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplySettings(TObject *Sender)
{
	WF1Resol->Position = WF1ResPos;
	ff1Size = FF1SMIN << WF1ResPos;
	WF2Resol->Position = WF2ResPos;
	ff2Size = FF2SMIN << WF2ResPos;
	FF1HzPerPixel = (float)srateIn / ff1Size;

	Volume->Position = vol;
	LockVol->Checked = vollocked;

	Avg1 = AvgWF1->Items->Strings[AvgWF1->ItemIndex].ToInt();
	Avg2 = AvgWF2->Items->Strings[AvgWF2->ItemIndex].ToInt();

	switch(mode)
	{
	  case AM     : mode = NONE; BAM->Click();  break;
	  case FM     : mode = NONE; BFM->Click();  break;
	  case ECSS   : mode = NONE; BECSS->Click();break;
	  case DRM    : mode = NONE; BDRM->Click(); break;
	  case USB    : mode = NONE; BUSB->Click(); break;
	  case LSB    : mode = NONE; BLSB->Click(); break;
	  case CW     : mode = NONE; BCW->Click();  break;
	}

	if(nr)   NRedMouseDown(Sender, mbLeft, ShiftState, 0, 0);
	if(cwp)  CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);
	if(nblanker) NBlankerMouseDown(Sender, mbLeft, ShiftState, 0, 0);
	if(wf1slow)
	  WF1Slow->Down = false;  // trick to force the following call to set it true
	else
	  WF1Slow->Down = true;   // trick to force the following call to set it false

	WF1SlowClick(Sender);

	SP1FSpeed->Down = sp1fullspeed;
	SP1FSpeed->DrawColor = sp1fullspeed ? clLight : clGray;
	BWF1Rev->Down = revWF1scroll;
	BWF1Rev->DrawColor = revWF1scroll ? clLight : clGray;
	BWF1Avg->Down = WF1avg;
	BWF1Avg->DrawColor = WF1avg ? clLight : clGray;
	BWF2Avg->Down = WF2avg;
	BWF2Avg->DrawColor = WF2avg ? clLight : clGray;

	int sr = srateIn;
	srateIn = 0;

	switch(sr)
	{
	  case  8000  : S8000InClick(Sender);   break;
	  case 11025  : S11025InClick(Sender);  break;
	  case 16000  : S16000InClick(Sender);  break;
	  case 22050  : S22050InClick(Sender);  break;
	  case 24000  : S24000InClick(Sender);  break;
	  case 32000  : S32000InClick(Sender);  break;
	  case 44100  : S44100InClick(Sender);  break;
	  case 48000  : S48000InClick(Sender);  break;
	  case 88200  : S88200InClick(Sender);  break;
	  case 96000  : S96000InClick(Sender);  break;
	  case 176400 : S176400InClick(Sender); break;
	  case 192000 : S192000InClick(Sender); break;
	}

	if(!srateIn) srateIn = sr;

	SP1GainChange(Sender);
	SP1BaseChange(Sender);

	switch(wintype)
	{
	  case Hann      : WHann->Checked      = true; break;
	  case Blackman  : WBlackman->Checked  = true; break;
	  case Hamming   : WHamming->Checked   = true; break;
	  case wsin3     : WSin3->Checked      = true; break;
	  case wsin5     : WSin5->Checked      = true; break;
	  case nuttall   : WNuttall->Checked   = true; break;
	  case nowin     : WNowin->Checked     = true; break;
	}

	switch(activePalette)
	{
	  case Spectran  : PalSpectran->Checked = true; break;
	  case Horne     : PalHorne->Checked    = true; break;
	  case SpecExt   : PalSpecExt->Checked  = true; break;
	  case Winrad    : PalWinrad->Checked   = true; break;
	  case Linrad    : PalLinrad->Checked   = true; break;
	}
	palettechanged = true;

	switch(driver)
	{
	  case WMME      : MWMME->Click(); break;
	  case ASIO      : MASIO->Click(); break;
	}

	switch(agc)
	{
	  case fast      : AgcFastClick(Sender); break;
	  case slow      : AgcSlowClick(Sender); break;
	  case off       : agc = slow; AgcSlowClick(Sender); break;
	}

    switch(datamode)
    {
      case mLeft      : MLeft->Click();      break;
	  case mRight     : MRight->Click();     break;
      case mLeftRight : MLeftRight->Click(); break;
	  case mIQ        : MIQ->Click();        break;
	}

	switch(polyfactor)
	{
	  case 1          : Poly1MouseDown(Sender, mbLeft, ShiftState, 0, 0);  break;
	  case 2          : Poly2MouseDown(Sender, mbLeft, ShiftState, 0, 0);  break;
	  case 4          : Poly4MouseDown(Sender, mbLeft, ShiftState, 0, 0);  break;
	}

	bpLocut = max(FF2locut, 50.F);
	bpHicut = min(FF2hicut, vsrOut/2 - 50.F);

	SwapIQ->Checked = swapIQ;
	MPrioNorm->Checked = processPriority == NORMAL_PRIORITY_CLASS;
	MPrioHi->Checked   = !MPrioNorm->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::QuitButtClick(TObject *Sender)
{
	if(running)
	  StartStopClick(Sender);

	if(oddRate) srateIn = 48000;
	SaveBP(Sender);

	SaveSettings("LastUsed");
	if(hextIO != NULL) CloseHW();
	delete IWav;
	delete OWav;
	delete myFont;
	BufFree();
	delete[] cosval;
	delete asioDrivers;
	delete[] ExtLoDigit;
	delete[] NCODigit;
	if(hKernel != NULL)  FreeLibrary(hKernel);
	if(hextIO != NULL)   FreeLibrary(hextIO);

// re-set Windows animation effects as they were initially
	if(prevAnim)
	{
	  ai.iMinAnimate = prevAnim;
	  SystemParametersInfo(SPI_SETANIMATION,sizeof(ANIMATIONINFO),&ai,0);
	}

	SystemParametersInfo(SPI_SETCOMBOBOXANIMATION, 0, &comboanim, 0);
	SystemParametersInfo(SPI_SETMENUANIMATION, 0, &menuanim, 0);
	SystemParametersInfo(SPI_SETTOOLTIPANIMATION, 0, &toolanim, 0);
//--------------------------------------------------

	Width = W - 10;;   Height = H - 10; // to avoid remnants in the task bar
	Application->Terminate();
}
//---------------------------------------------------------------------------
// Manage here the selection of the sound card
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelCardButtClick(TObject *Sender)
{
	bool wasrunning = running;
	int  devIdSaveIn, devIdNewIn, devIdSaveOut, devIdNewOut, asioNumSave;
	char devNameSaveIn[128], devNameSaveOut[128], asioNameSave[128];

	if(SelRate->Down) SelRateClick(Sender);

	devIdSaveIn = devIdIn; devIdSaveOut = devIdOut;
	asioNumSave = asioNum;
	strcpy(devNameSaveIn,  devNameIn);
	strcpy(devNameSaveOut, devNameOut);
	strcpy(asioNameSave,   asioName);

	if(driver == WMME)
	  if(SelCardForm->ShowModal() == mrOk)
	  {
		PutCurrentCard();    // write info into the registry

		if(devIdIn != devIdSaveIn || devIdOut != devIdSaveOut)
		{
		  SetParam(Sender);      // only to reset the output sample rate
		  if(wasrunning)
		  {
			 StartStopClick(Sender);   // stop...
			 Sleep(100);
			 StartStopClick(Sender);   // and restart
		  }
		}
	  }

	if(driver == ASIO)
	{
	  if(SelASIOForm->ShowModal() == mrOk)
	  {
		PutCurrentCard();    // write info into the registry

		if(asioNumSave != asioNum)
		  if(wasrunning)
		  {
			 StartStopClick(Sender);   // stop...
			 Sleep(100);
			 StartStopClick(Sender);   // and restart
		  }
	  }
	}
}
//---------------------------------------------------------------------------
void __fastcall fatal(char * msg)
{
   MessageBox(MainForm->hwnd, msg,"digiRadio",
								   MB_APPLMODAL | MB_OK | MB_ICONEXCLAMATION);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelRateClick(TObject *Sender)
{
   if(SelRate->Down)
   {
	 RatePanel->Visible = false;
	 SelRate->Font->Color = clPesca;
	 SelRate->Down = false;
   }
   else
   {
	 if(driver == ASIO)
	 {
	   if(!running) CheckASIO();
	   ShowASIOCP->Visible = true;
	   LAsio->Visible = true;
	   LControlPanel->Visible = true;
	 }
	 else
	 {
	   ShowASIOCP->Visible = false;
	   LAsio->Visible = false;
	   LControlPanel->Visible = false;
	 }
	 RatePanel->Visible = true;
	 SelRate->Font->Color = clWhite;
	 SelRate->Down = true;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S8000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 8000 || mode == DRM || mode == FM) return;
   if(running) StartStopClick(Sender);
   srateIn = 8000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S11025InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 11025 || mode == DRM || mode == FM) return;
   if(running) StartStopClick(Sender);
   srateIn = 11025;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S16000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 16000 || mode == DRM || mode == FM) return;
   if(running) StartStopClick(Sender);
   srateIn = 16000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S22050InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 22050 || mode == DRM || mode == FM) return;
   if(running) StartStopClick(Sender);
   srateIn = 22050;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S24000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 24000) return;
   if(running) StartStopClick(Sender);
   srateIn = 24000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S32000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 32000) return;
   if(running) StartStopClick(Sender);
   srateIn = 32000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S44100InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 44100) return;
   if(running) StartStopClick(Sender);
   srateIn = 44100;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S48000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 48000) return;
   if(running) StartStopClick(Sender);
   srateIn = 48000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S88200InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 88200) return;
   if(running) StartStopClick(Sender);
   srateIn = 88200;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S96000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 96000) return;
   if(running) StartStopClick(Sender);
   srateIn = 96000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S176400InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 176400) return;
   if(running) StartStopClick(Sender);
   srateIn = 176400;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S192000InClick(TObject *Sender)
{
   bool wasrunning = running;

   if(srateIn == 192000) return;
   if(running) StartStopClick(Sender);
   srateIn = 192000;
   SetParam(Sender);
   if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SOtherInClick(TObject *Sender)
{
   SOtherIn->Down = true;
   srateIn = OtherRate->Caption.ToInt();
   SetParam(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MinimizeClick(TObject *Sender)
{
   if(controllingExtHW && NULL != HideGUI) HideGUI();
   Application->Minimize();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF1ScaleMouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbLeft)  movingscale1 = true;
	if(Button == mbRight) WF1resizing  = true;
	Xdown = X;
	GetCursorPos(&mousePos);
	Ydown = mousePos.y;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF1ScaleMouseMove(TObject *Sender,
	  TShiftState Shift, int X, int Y)
{
	if(movingscale1)
	{
	   float deltaF = (Xdown - X) * FF1HzPerPixel;
	   if(FF1InizioScala + deltaF <= srateIn/2. - FF1span &&
             FF1InizioScala + deltaF >= ((datamode == mIQ) ? (-srateIn/2.) : 0.F))
       {  FF1InizioScala += deltaF;
		      FF1Scale->StartVal = (FF1InizioScala + extLOfreq + extLOoffset) / 1000.;
          FF1Scale->StopVal  = FF1Scale->StartVal + FF1span / 1000.;
          FF1Scale->Repaint();

		      if(controllingExtHW && IFLimitsChanged != NULL)
				IFLimitsChanged(FF1InizioScala + extLOfreq, FF1InizioScala + extLOfreq + FF1span);
	   }
	   Xdown = X;

	   AdjustBlueWindow();
	}

	if(!WF1resizing) return;

	GetCursorPos(&mousePos);
	int delta = mousePos.y - Ydown;

	WF1->Height     += delta;
	SP1->Height     -= delta;
	if(SP1->Height  < 2 || WF1->Height < 2) goto update;

	WF1H            = WF1->Height;
	FF1Scale->Top   = WF1T + WF1H;
	SP1->Top        = FF1Scale->Top + FF1Scale->Height;
	SP1T            = SP1->Top;     // referred to screen
	SP1H            = SP1->Height;

	WF1bm->Height   = WF1H;

	SP1rect         = Rect(0,0,SP1W, SP1H);
	WF1rect         = Rect(0,0,WF1W, WF1H);
	WF1dest         = Rect(0, 1, WF1W, WF1H);
	WF1src          = Rect(0, 0, WF1W, WF1H - 1);
	FF1rect         = Rect(WF1L-1, WF1T-1, WF1L + WF1W + 2,
						   WF1T + WF1H + FF1Scale->Height + SP1H + 2);
	WF1->Fill(WF1bm);

	WF1bm->Canvas->FillRect(WF1rect);

	SelectObject(SP1bmhdc, SP1bm);

	if(!running) ::FillRect(SP1bmhdc, &SP1rect, SP1back);
update:
	GetCursorPos(&mousePos);
	Ydown = mousePos.y;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF1ScaleMouseUp(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
    movingscale1 = WF1resizing = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF2ScaleMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if(Button == mbLeft)  movingscale2 = true;
    if(Button == mbRight) WF2resizing  = true;
    Xdown = X;
    GetCursorPos(&mousePos);
    Ydown = mousePos.y;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF2ScaleMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
	if(movingscale2)
    {
       float deltaF = (Xdown - X) * FF2HzPerPixel;
       if(FF2InizioScala + deltaF <= vsrOut/2. - FF2span &&
                                     FF2InizioScala + deltaF >= 0.)
       {  FF2InizioScala += deltaF;
          FF2Scale->StartVal = FF2InizioScala;
          FF2Scale->StopVal  = FF2InizioScala + FF2span;
          FF2Scale->Repaint();
	   }
       Xdown = X;
    }

    if(!WF2resizing) return;

    GetCursorPos(&mousePos);
    int delta = mousePos.y - Ydown;

    WF2->Height     += delta;
    SP2->Height     -= delta;
    if(SP2->Height  < 2 || WF2->Height < 2) goto update;

    WF2H            = WF2->Height;
    FF2Scale->Top   = WF2T + WF2H;
    SP2->Top        = FF2Scale->Top + FF2Scale->Height;
    SP2T            = SP2->Top;     // referred to screen
    SP2H            = SP2->Height;

    WF2bm->Height   = WF2H;

	SP2rect         = Rect(0,0,SP2W, SP2H);
    WF2rect         = Rect(0,0,WF2W, WF2H);
    WF2dest         = Rect(0, 1, WF2W, WF2H);
	WF2src          = Rect(0, 0, WF2W, WF2H - 1);
    FF2rect         = Rect(WF2L-1, WF2T-1, WF2L + WF2W + 2,
                           WF2T + WF2H + FF2Scale->Height + SP2H + 2);
    WF2->Fill(WF2bm);

    WF2bm->Canvas->FillRect(WF2rect);

    SelectObject(SP2bmhdc, SP2bm);

    if(!running) ::FillRect(SP2bmhdc, &SP2rect, SP2back);
update:
    GetCursorPos(&mousePos);
    Ydown = mousePos.y;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FF2ScaleMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    movingscale2 = WF2resizing = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1GainChange(TObject *Sender)
{
	SP1range       = 170 - SP1Gain->Position;     //  160..10
    SP1toprange    = SP1bottomrange + SP1range;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1BaseChange(TObject *Sender)
{
    SP1bottomrange = SP1baselevel = SP1Base->Position - 160;  //  00..-160
    SP1toprange    = SP1bottomrange + SP1range;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AppMessage(tagMSG &Msg, bool &Handled)
{
	if (Msg.hwnd == hwnd && Msg.message == WM_PAINT)
	{
	  FormPaint(this);
	  Handled = true;
	}
// for all other messages, Handled remains False
// so that other message handlers can respond
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AppRestore()
{
   if(controllingExtHW && NULL != ShowGUI) ShowGUI();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormPaint(TObject *Sender)
{
	EnterCriticalSection(&screenUpdate);
	  MainForm->Canvas->FrameRect(FF1rect);
	  MainForm->Canvas->FrameRect(FF2rect);
	  MainForm->Update();
	  HDC hdcform = GetDC(hwnd);
	  BitBlt(hdcform, SP1L, SP1T, SP1W, SP1H, SP1bmhdc, 0, 0, SRCCOPY);
	  BitBlt(hdcform, SP2L, SP2T, SP2W, SP2H, SP2bmhdc, 0, 0, SRCCOPY);
	  if(inSP1) DynWrite();
	  ReleaseDC(hwnd, hdcform);
	LeaveCriticalSection(&screenUpdate);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1MouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	static int oldX = -1, oldY = -1;
	float t, delta;
	int it;

	if(Button != mbLeft) return;

	if(X == oldX && Y == oldY) return;
	oldX = X;
	oldY = Y;

	if(inBetween)
	{
	  Xdown = X;
	  offs = X2F1(X) - FF1freq;
	  SP1drag = true;
	  return;
	}

	if(mode == CW)
	  FF1freq = X2F1(X);
	else
	{
	  t = X2F1(X) + extLOfreq;
	  it = t / 1000. + 0.5F;  it *= 1000;
	  if(fabs(t - (float)it) < 400.) FF1freq = it - extLOfreq;
	  else  FF1freq = t - extLOfreq;
	}

	AdjustBlueWindow();
	zapneeded = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP2MouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbLeft)
	{ Ldrag  = Lnear;
	  Hdrag  = Hnear;
	  Xdown  = X;
	  if(mode != CW || Ldrag || Hdrag) return;
	}

	if(mode == CW)
	  if(Shift.Contains(ssCtrl)) bfoChanging = true;
	  else WF2MouseDown(Sender, Button, Shift, X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP2MouseUp(TObject *Sender, TMouseButton Button,
	  TShiftState Shift, int X, int Y)
{
	Lnear = Hnear = Ldrag = Hdrag = false;

	if(bfoChanging)
	{
	  bfoChanging = false;
	  CWPitch = X2F2(X);

	  FF2cf    = CWPitch;
	  bpLocut  = FF2locut = FF2cf - bw/2;
	  bpHicut  = FF2hicut = FF2cf + bw/2;
	  SetParam(Sender);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP2MouseMove(TObject *Sender, TShiftState Shift,
	  int X, int Y)
{
	float oldLo, oldHi, delta;

	if(bfoChanging)
	{
	  CWPitch = X2F2(X);
	  bpchanged = true;

	  FF2cf    = CWPitch;
	  return;
	}

	if(!(Ldrag || Hdrag) || X < 0) return;

	if(Ldrag)
	{
	  oldLo = FF2locut;
	  FF2locut = max(X2F2(X), 50.F);
	  bpLocut = FF2locut = min(FF2cf - 5., FF2locut);
	}
	else
	 if(Hdrag)
	 {
	   oldHi = FF2hicut;
	   FF2hicut = min(X2F2(X), vsrOut/2. - 50.F);
	   bpHicut = FF2hicut = max(FF2cf + 5., FF2hicut);
	 }

	if(mode == CW)
	{
	  if(Ldrag)
	  {
		delta = FF2locut - oldLo;
		FF2hicut = FF2hicut - delta;
	  }
	  else
	   if(Hdrag)
	   {
		 delta = FF2hicut - oldHi;
		 FF2locut = FF2locut - delta;
	   }

	  bpHicut = min(FF2hicut, vsrOut/2. - 50.F);
	  bpLocut = max(FF2locut, 50.F);
	  bw = bpHicut - bpLocut;
	}
	else if(mode != DRM)
	{
	  SSBFF2locut = FF2locut;
	  SSBFF2hicut = FF2hicut;
	}

	AdjustBlueWindow();
	FF2cf = (FF2locut + FF2hicut) / 2.;
	bpchanged = true;
	Sleep(0);               // relinquish time slice
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF2MouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	float fpeak, fdelta;

	if(mode != CW) return;

	fpeak = FF2InizioScala + X * FF2HzPerPixel; // from index to actual frequency
	fdelta = CWPitch - fpeak;                   // how far from bandpass center ?
	FF1freq -= fdelta;                          // correct the LO frequency
	AdjustBlueWindow();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WHannClick(TObject *Sender)
{
	wintype = Hann;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WBlackmanClick(TObject *Sender)
{
	wintype = Blackman;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WHammingClick(TObject *Sender)
{
	wintype = Hamming;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WSin3Click(TObject *Sender)
{
	wintype = wsin3;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WSin5Click(TObject *Sender)
{
    wintype = wsin5;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WNuttallClick(TObject *Sender)
{
    wintype = nuttall;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WNowinClick(TObject *Sender)
{
    wintype = nowin;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PalSpectranClick(TObject *Sender)
{
    activePalette  = Spectran;
    palettechanged = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PalHorneClick(TObject *Sender)
{
    activePalette  = Horne;
    palettechanged = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PalSpecExtClick(TObject *Sender)
{
	activePalette  = SpecExt;
	palettechanged = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PalWinradClick(TObject *Sender)
{
    activePalette  = Winrad;
    palettechanged = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PalLinradClick(TObject *Sender)
{
    activePalette  = Linrad;
    palettechanged = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AgcOnOffClick(TObject *Sender)
{
	// Shield for wfm
	if (agc==off && mode==FM && submode!=lower) return;
	
	if(agc == off)
	{
	  agc = agcsave;
	  LThr->Caption = "Thr";
	}
	else
	{
	  agcsave = agc;
	  agc = off;
	  LThr->Caption = "Lev";
	}

	LAGC->Caption = (agc == off) ? "AGC Off" : "AGC On";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AgcFastClick(TObject *Sender)
{
	if(agc == off) return;
	AgcFast->Down = true;  AgcSlow->Down = false;
	agcsave = agc = fast;
	agc_decay = 0.998;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AgcSlowClick(TObject *Sender)
{
	if(agc == off) return;
	AgcFast->Down = false;  AgcSlow->Down = true;
    agcsave = agc = slow;
    agc_decay = 0.9999;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MWMMEClick(TObject *Sender)
{
    bool wasrunning = running;

    MWMME->Checked = true;
	if(driver == WMME) return;
	if(running) StartStopClick(Sender);
    driver = WMME;
	mustChoose = true;

    SelCardButtClick(Sender);
	mustChoose = false;

	S8000In->Enabled   = true;
	S11025In->Enabled  = true;
	S16000In->Enabled  = true;
	S22050In->Enabled  = true;
	S24000In->Enabled  = true;
	S32000In->Enabled  = true;
	S44100In->Enabled  = true;
	S48000In->Enabled  = true;
	S88200In->Enabled  = true;
	S96000In->Enabled  = true;
	S176400In->Enabled = true;
	S192000In->Enabled = true;

    SetParam(Sender);

	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MASIOClick(TObject *Sender)
{
	if(numASIOdrv < 1)    // set at program initialization
	{
	   warn("Attention - No ASIO drivers found on this PC. ASIO cannot be selected");
	   return;
	}

    bool wasrunning = running;

	MASIO->Checked  = true;
	S8000Out->Down  = false;
	S11025Out->Down = false;
	S24000Out->Down = false;
	if(driver == ASIO) return;
	if(running) StartStopClick(Sender);
	driver = ASIO;
	mustChoose = true;

	SelCardButtClick(Sender);
	mustChoose = false;
	SetParam(Sender);
	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1MouseUp(TObject *Sender, TMouseButton Button,
	  TShiftState Shift, int X, int Y)
{
	movingscale1 = WF1resizing = SP1drag = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF1MouseUp(TObject *Sender, TMouseButton Button,
	  TShiftState Shift, int X, int Y)
{
	movingscale1 = WF1resizing = SP1drag = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1MouseMove(TObject *Sender, TShiftState Shift,
	  int X, int Y)
{
	static int oldX = -1, oldY = -1;

	if(WF1resizing)
	{ FF1ScaleMouseMove(Sender, Shift, X, Y);
	  return;
	}

	if(X == oldX && Y == oldY) return;
	oldX = X;
	oldY = Y;

	if(!SP1drag) return;

	FF1freq = X2F1(X) - offs;

	AdjustBlueWindow();
	Sleep(0);               // relinquish time slice
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF1MouseMove(TObject *Sender, TShiftState Shift,
	  int X, int Y)
{
	if(WF1resizing) FF1ScaleMouseMove(Sender, Shift, X, Y);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormMouseUp(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	movingscale1 = movingscale2 = WF1resizing = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowOptionsClick(TObject *Sender)
{
	OpMenu->Popup(8, 25);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF1ResolClick(TObject *Sender, TUDBtnType Button)
{
	bool wasrunning = running;
	int oldpos = WF1ResPos;

	WF1ResPos = WF1Resol->Position;
	if(WF1ResPos == oldpos) return;

	if(driver == ASIO && downFactor == 1)
	  if(wasrunning) StartStopClick(Sender);
	  else;
	else
	  if(running) PFF1->Suspend();

	if(datamode == mIQ)
	   ff1Size = FF1SMIN << WF1ResPos;
	else
	   ff1Size = (FF1SMIN * 2) << WF1ResPos;
	SetParam(Sender);

	if(driver == ASIO && downFactor == 1)
	  if(wasrunning) StartStopClick(Sender);
	  else;
	else
	  if(running) PFF1->Resume();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SP2GainChange(TObject *Sender)
{
	SP2range       = 170 - SP2Gain->Position;     //  160..10
	SP2toprange    = SP2bottomrange + SP2range;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SP2BaseChange(TObject *Sender)
{
	SP2bottomrange = SP2baselevel = SP2Base->Position - 160;  //  00..-160
	SP2toprange    = SP2bottomrange + SP2range;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF2ResolClick(TObject *Sender, TUDBtnType Button)
{
	bool wasrunning = running;
	int oldpos = WF2ResPos;

	WF2ResPos = WF2Resol->Position;
	if(WF2ResPos == oldpos) return;

	if(driver == ASIO && downFactor == 1)
	  if(wasrunning) StartStopClick(Sender);
	  else;
	else
	  if(running) PFF1->Suspend();

	ff2Size = FF2SMIN << WF2ResPos;
	SetParam(Sender);

	if(driver == ASIO && downFactor == 1)
	  if(wasrunning) StartStopClick(Sender);
	  else;
	else
	  if(running) PFF1->Resume();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MLeftClick(TObject *Sender)
{
	bool wasrunning = running;

	MLeft->Checked = true;

	if(running) StartStopClick(Sender);
	datamode = mLeft;
	WF1Resol->Max = 3;
	if(WF1Resol->Position == 4) WF1Resol->Position = 3;
	ff1Size = (FF1SMIN * 2) << WF1ResPos;
	SetParam(Sender);
	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MRightClick(TObject *Sender)
{
	bool wasrunning = running;

	MRight->Checked = true;

	if(running) StartStopClick(Sender);
	datamode = mRight;
	WF1Resol->Max = 3;
	if(WF1Resol->Position == 4) WF1Resol->Position = 3;
	ff1Size = (FF1SMIN * 2) << WF1ResPos;
	SetParam(Sender);
	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MLeftRightClick(TObject *Sender)
{
	bool wasrunning = running;

	MLeftRight->Checked = true;

	if(running) StartStopClick(Sender);
	datamode = mLeftRight;
	WF1Resol->Max = 3;
	if(WF1Resol->Position == 4) WF1Resol->Position = 3;
	ff1Size = (FF1SMIN * 2) << WF1ResPos;
	SetParam(Sender);
	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MIQClick(TObject *Sender)
{
	bool wasrunning = running;

	MIQ->Checked = true;

	if(running) StartStopClick(Sender);
	datamode = mIQ;
	WF1Resol->Max = 4;
	ff1Size = FF1SMIN << WF1ResPos;
	SetParam(Sender);
	if(wasrunning) StartStopClick(Sender);
}
//--------------------------------------------------------------------------
void __fastcall TMainForm::BFMClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	//if(mode == FM) return;
	if (mode != FM) submode = lower;
	else {
		// Next wideband submodes
		switch (submode) {
			case lower:
				submode = upper;
				break;
			case upper:
				submode = both;
				break;
			case both:
				submode = lower;
				break;
		default:
			;
		}

		// Setup wide mode gui
		switch (submode) {
			case lower:
				EnableWFMGUI(false);
				break;
			case upper:
			case both:
				Label2->Caption = (submode==upper)?"WFM":"WFMStereo";

				EnableWFMGUI(true);
				wfm_deemph_tau = atof(String(ComboBox1->Text).c_str()) / 1.e6;
				wfm_rf_cut = atoi(String(ComboBox2->Text).c_str()) * 1000;
				wfm_audio_cut = atoi(String(ComboBox3->Text).c_str()) * 1000;

			break;
		default:
			;
		}

    }
	DenoiserInitialized = false;

	if(running && mode != DRM) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = FM;
	RestoreBP(Sender);

	rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode != DRM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BFM->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('F');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnableWFMGUI(bool enable)
{

	if (enable) {
		if (agc != off) {
			AgcOnOffClick(this);
        }
		Label2->Visible = true;
		ComboBox1->Visible = true;
		ComboBox2->Visible = true;
		ComboBox3->Visible = true;
		Label1->Visible = true;
		Label3->Visible = true;
		Label4->Visible = true;
		Label6->Visible = true;
		Label7->Visible = true;
		Label8->Visible = true;
		Label9->Visible = true;
		VU->Caption = String("Disabled");
		VU->Enabled = false;
		VU->Position = 1;
	}
	else {
		Label2->Visible = false;
		ComboBox1->Visible = false;
		ComboBox2->Visible = false;
		ComboBox3->Visible = false;
		Label1->Visible = false;
		Label3->Visible = false;
		Label4->Visible = false;
		Label6->Visible = false;
		Label7->Visible = false;
		Label8->Visible = false;
		Label9->Visible = false;
		VU->Caption = String("S-units");
		VU->Enabled = true;
	}
	
	VU->Update();
	VU->Refresh();

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BUSBClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == USB) return;
	EnableWFMGUI(false);
	DenoiserInitialized = false;

	if(running && (mode == DRM || mode == FM)) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = USB;
	RestoreBP(Sender);

	rxband.left   = (FF1freq + FF2locut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode == DRM || prevmode == FM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BUSB->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('U');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BLSBClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == LSB) return;
	EnableWFMGUI(false);
	DenoiserInitialized = false;

	if(running && (mode == DRM || mode == FM)) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = LSB;
	RestoreBP(Sender);

	rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq - FF2locut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode == DRM || prevmode == FM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BLSB->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('L');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BCWClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == CW) return;
	EnableWFMGUI(false);
	DenoiserInitialized = false;

	if(running && (mode == DRM || mode == FM)) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = CW;
	RestoreBP(Sender);

	rxband.left   = (FF1freq - (FF2hicut - FF2locut)/2 - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + (FF2hicut - FF2locut)/2 - FF1InizioScala)/FF1HzPerPixel;

	AgcFastClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode == DRM || prevmode == FM)
	{
	   Sleep(500);
	   StartStopClick(Sender);
	}

	BCW->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('C');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BAMClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == AM) return;

	EnableWFMGUI(false);

	DenoiserInitialized = false;

	if(running && (mode == DRM || mode == FM)) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = AM;
	RestoreBP(Sender);

	rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode == DRM || prevmode == FM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BAM->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('A');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BECSSClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == ECSS) return;
	EnableWFMGUI(false);


	DenoiserInitialized = false;

	if(running && (mode == DRM || mode == FM)) StartStopClick(Sender);

	SaveBP(Sender);
	prevmode = mode;
	mode = ECSS;
	RestoreBP(Sender);

	rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);
	SetParam(Sender);

	if(!wasrunning) DoWF1(NULL);
	else if(prevmode == DRM || prevmode == FM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BECSS->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('E');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BDRMClick(TObject *Sender)
{
	bool wasrunning = running;
	runmode prevmode;

	if(mode == DRM) return;
	if(running && mode != FM) StartStopClick(Sender);
	EnableWFMGUI(false);


	SaveBP(Sender);
	prevmode = mode;
	mode = DRM;
	RestoreBP(Sender);

	rxband.left   = (FF1freq + FF2locut - FF1InizioScala)/FF1HzPerPixel;
	rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;

	if(CWPk->Down)
	   CWPkMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BAFC->Down)
	   BAFCMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(NRed->Down)
	   NRedMouseDown(Sender, mbLeft, ShiftState, 0, 0);

	if(BDespread->Down)
	   BDespread->Click();

	AgcSlowClick(Sender);

	SetParam(Sender);
	if(!wasrunning) DoWF1(NULL);
	else if(prevmode != FM)
	{
	   Sleep(200);
	   StartStopClick(Sender);
	}

	BDRM->Down = true;
	if(ModeChanged != NULL && !mustGetMode) ModeChanged('D');
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveBP(TObject *Sender)
{
	switch(mode)
	{
		case NONE  : return;
		case CW    :
					 CWbw = bw;
					 CWWF2Resol = WF2Resol->Position;
					 break;
		case LSB   :
		case USB   :
		case AM    :
		case ECSS  :
					 SSBFF2locut = FF2locut;
					 SSBFF2hicut = FF2hicut;
					 SSBWF2Resol = WF2Resol->Position;
					 break;
		case FM    :
					 FMFF2locut = FF2locut;
					 FMFF2hicut = FF2hicut;
					 FMWF2Resol = WF2Resol->Position;
					 break;
		case DRM   :
					 DRMFF2locut = FF2locut;
					 DRMFF2hicut = FF2hicut;
					 DRMWF2Resol = WF2Resol->Position;
					 break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RestoreBP(TObject *Sender)
{
	switch(mode)
	{
		case CW    :
					 FF2cf    = CWPitch;
					 bw       = CWbw;
					 FF2locut = bpLocut = FF2cf - bw/2.;
					 FF2hicut = bpHicut = FF2cf + bw/2.;
					 WF2Resol->Position = CWWF2Resol;
					 break;
		case LSB   :
		case USB   :
		case AM    :
		case ECSS  :
					 FF2locut = bpLocut = SSBFF2locut;
					 FF2hicut = bpHicut = SSBFF2hicut;
					 WF2Resol->Position = SSBWF2Resol;
					 break;
		case FM    :
					 FF2locut = bpLocut = FMFF2locut;
					 FF2hicut = bpHicut = FMFF2hicut;
					 WF2Resol->Position = FMWF2Resol;
					 break;

		case DRM   :
					 FF2locut = bpLocut = DRMFF2locut;
					 FF2hicut = bpHicut = DRMFF2hicut;
					 WF2Resol->Position = DRMWF2Resol;
					 break;
	}

	WF2ResolClick(Sender, (TUDBtnType) NULL);
}
//---------------------------------------------------------------------------
float inline __fastcall X2F2(int X)
{
	return MainForm->FF2InizioScala + X * MainForm->FF2HzPerPixel;
}
//---------------------------------------------------------------------------
float inline __fastcall X2F1(int X)
{
	return MainForm->FF1InizioScala + X * MainForm->FF1HzPerPixel;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AGCGainChange(TObject *Sender)
{
	agcthr  = 1e-5 + 60.e-5 - 30.e-5 * log10(1 + AGCGain->Position);
	agcgain = AGCGain->Position / 99.;
	agcgain = pow(agcgain, 2.2) * 2.;

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ResDefClick(TObject *Sender)
{
	bool wasrunning = running;

	if(running) StartStopClick(Sender);
	LoadSettings("*");
    ApplySettings(Sender);
	SetParam(Sender);
	SelSoundCard->Click();
	if(wasrunning) StartStopClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WStopClick(TObject *Sender)
{
	wlooping = false;
	SelSoundCard->Checked = true;  // otherwise the check in that routine will skip 
	SelSoundCardClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelMixerClick(TObject *Sender)
{
    sprintf(s, "/d%d /r", devIdIn);
    spawnlp(P_NOWAITO, "sndvol32.exe","sndvol32.exe", s, NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SelWavClick(TObject *Sender)
{
	char *p, name[256];

	WavOpenDialog->InitialDir = wavdir;
	WavOpenDialog->FileName = wavname;
	if(!WavOpenDialog->Execute()) return;

    if(wavin)
	{ if(running) StartStopClick(Sender);
	  IWav->InClose();   // should a previous file be still open
      wavin = false;
    }
    strcpy(name, WavOpenDialog->FileName.c_str());

	wavdir  = ExtractFilePath(WavOpenDialog->FileName);
	wavname = ExtractFileName(WavOpenDialog->FileName);

	if((p = strrchr(name, '.')) == NULL)
	   strcat(name, ".wav");
	else
	   strcpy(p, ".wav");

	if(!IWav->InOpen(name))
	{
	   sprintf(s, "File %s could not be opened for reading", name);
	   warn(s);
	   WStopClick(Sender);
	   return;
	}

	if(controllingExtHW && NULL != HideGUI) HideGUI();

	controllingExtHW = false;
	MASIO->Visible = false;
	MWMMEClick(Sender);
	EoF = false;

	if(running) StartStopClick(Sender);

	IWav->SetSampling();
	InRates->Enabled = false;
	wavin = paused = true;

	WPanel->Visible = true;
	WTrack->Position = 0;
	WavName->Caption = wavname;
	WavName->Visible = true;
	WavTime->Visible = true;
	StartStop->Visible = false;
	SelWav->Checked = true;

	DisplayWavTime();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WStartClick(TObject *Sender)
{
	if(!paused)
	{
	  wlooping = false;
	  return;
	}

    paused = false;
	SetParam(Sender);
	if(!running)
	  StartStopClick(Sender);
    else
    {
      if(driver == WMME)
	  { PIn->wreadlen = BUFSIZE;
		IWav->InRead(PIn->inL, PIn->inR, PIn->wreadlen);
        inQued = 1;
      }
	  SetEvent(InBufferReady);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WLoopClick(TObject *Sender)
{
	WStartClick(Sender);
    wlooping = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WPauseClick(TObject *Sender)
{
	paused = true;
	if(driver == WMME) ResetOut();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WBackClick(TObject *Sender)
{
	if(running && driver == WMME) StartStopClick(Sender);
	paused = true;
	IWav->InRewind();
	EoF = false;
	WStartClick(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AvgWF1Change(TObject *Sender)
{
	Avg1 = AvgWF1->Items->Strings[AvgWF1->ItemIndex].ToInt();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AvgWF2Change(TObject *Sender)
{
	Avg2 = AvgWF2->Items->Strings[AvgWF2->ItemIndex].ToInt();
}
//---------------------------------------------------------------------------
// Intellimouse support
void __fastcall TMainForm::WheelProc(Messages::TMessage &msg)
{
	int  idx, rc;
	static int oldfreq = 0;

	for(int k = 0; k < NUMDIGITS; k++)
	{
	  if(inExtLoDig[k] || inNCODig[k]) idx = k;
	}

	if(inLO)
	{
	  if(wavin) return;      // during wav playback it has no meaning to change the LO
	  if(LOlocked) return;   // ExtIO DLL does not currently accept LO changes

	  if(msg.WParamHi & 0x1000)    // wheel up
		ExtLoDigit[idx].Inc();
	  else                         // wheel down
		ExtLoDigit[idx].Dec();

	  extLOfreq = GetLONixieFreq();
	  if(controllingExtHW)
	  {
		rc = SetHWLO((long)extLOfreq);
	  }

	  FF1Scale->StartVal = (FF1InizioScala + extLOfreq) / 1000.;
	  FF1Scale->StopVal  = FF1Scale->StartVal + FF1span / 1000.;
	  FF1Scale->Repaint();

	  if(controllingExtHW && IFLimitsChanged != NULL)
		IFLimitsChanged(FF1InizioScala + extLOfreq, FF1InizioScala + extLOfreq + FF1span);

	  if(extLOfreq != oldLO && controlKey)
	  {
		SetNCONixieFreq(FF1freq + oldLO);
		oldLO = extLOfreq;
		goto update_NCO;
	  }
	  oldLO = extLOfreq;
	}

	else
	{
	  if(inNCO)
	  {
		if(msg.WParamHi & 0x1000)    // wheel up
		  NCODigit[idx].Inc();
		else                         // wheel down
		  NCODigit[idx].Dec();

update_NCO:
		FF1freq = GetNCONixieFreq() - extLOfreq;

	  }
	  else
	  { if(msg.WParamHi & 0x1000)     FF1freq += 10.;
		else                          FF1freq -= 10.;

		if(oldfreq != FF1freq + extLOfreq)
		{
		  oldfreq = FF1freq + extLOfreq;
		  SetNCONixieFreq(FF1freq + extLOfreq);
		  for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
		}
	  }
	  if(FF1freq > hibound)
	  {
		FF1freq = hibound;
		SetNCONixieFreq(FF1freq + extLOfreq);
		for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
	  }

	  if(FF1freq < lobound)
	  {
		FF1freq = lobound;
		SetNCONixieFreq(FF1freq + extLOfreq);
		for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
	  }
	}
	AdjustBlueWindow();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainWheelProc(Messages::TMessage &msg)
{
   if(msg.Msg == WM_MOUSEWHEEL)
   {
	 WheelProc(msg);
	 return;
   }

   MainOldProc(msg);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Avg1WheelProc(Messages::TMessage &msg)
{
   if(msg.Msg == WM_MOUSEWHEEL)
   {
	 WheelProc(msg);
	 return;
   }

   Avg1OldProc(msg);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Avg2WheelProc(Messages::TMessage &msg)
{
   if(msg.Msg == WM_MOUSEWHEEL)
   {
	 WheelProc(msg);
	 return;
   }

   Avg2OldProc(msg);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BZAPClick(TObject *Sender)
{
	if(mode != CW) return;

	if(BZAP->Down)
	  if(Sender == NULL)  // if Sender == NULL, the call is coming from Functions2
	  {
		zap = false;
		BZAP->Down = false;
	  }
	  else return;       // prevent double invokation when pressing Z key
	else
	{
	  zap = true;
	  afc = false;
	  BZAP->Down = true;
	  BAFC->Down = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BAFCMouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button != mbLeft) return;
	if(mode != CW) return;

	if(BAFC->Down)
	{
	  afc = false;
	  BAFC->Down = false;
	}
	else
	{
	  zap = false;
	  afc = true;
	  BZAP->Down = false;
	  BAFC->Down = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NRedMouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if(Button == mbRight)
    {
      Den1AdjForm->Show();
	  return;
	}
    if(Button != mbLeft) return;

    if(NRed->Down)
    {
      nr = false;
      NRed->Down = false;
    }
	else
    {
      nr = true;
      NRed->Down = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CWPkMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbRight)
    {
      Den2AdjForm->Show();
      return;
    }
	if(Button != mbLeft) return;

    if(CWPk->Down)
    {
	  cwp = false;
	  CWPk->Down = false;
    }
    else
	{
	  if(mode != CW) return;       // Denoiser 2 is for CW only...
	  cwp = true;
	  CWPk->Down = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SkewCalClick(TObject *Sender)
{
    SkewForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SwapIQClick(TObject *Sender)
{
    swapIQ = SwapIQ->Checked;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NBlankerMouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if(Button == mbRight)
	{
	  BlkAdjForm->Show();
	  return;
	}
	if(Button != mbLeft) return;

	if(NBlanker->Down)
	{
	  nblanker = false;
	  NBlanker->Down = false;
	}
	else
	{
	  nblanker = true;
	  NBlanker->Down = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AboutButtonClick(TObject *Sender)
{
	AboutForm->ShowModal();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HourTimerTimer(TObject *Sender)
{
	char timenow[80];

    strcpy(timenow, DateTimeToStr(Now()).c_str());
    Clock->Caption = timenow;
	Clock->Repaint();

    if(isXPorGT && getCpuLoad(cpuTot, cpuProc))
    {
	  CPUT->Position = cpuTot;
      CPUW->Position = cpuProc;
      wsprintf(s, "Total  (%d%%)", cpuTot);
	  LCPUT->Caption = s;
      wsprintf(s, "digiRadio (%d%%)", cpuProc);
      LCPUW->Caption = s;
      if(cpuTot > 95)  CPUT->BarColor = clRed;
      else             CPUT->BarColor = clYellow;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::VolumeChange(TObject *Sender)
{
    vol = Volume->Position;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BFullClick(TObject *Sender)
{
	if(mode != CW) fullview = false;
	else           fullview = !fullview;

    BFull->DrawColor = fullview ? clLight : clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF1SlowClick(TObject *Sender)
{
    if(WF1Slow->Down)
    {
	  WF1Slow->Font->Color = clGray;
	  wf1slow = false;
      WF1Slow->Down = false;
    }
    else
	{
      WF1Slow->Font->Color = clWhite;
      wf1slow = true;
      WF1Slow->Down = true;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF1ContrastChange(TObject *Sender)
{
    wf1gamma = (float)WF1Contrast->Position / 100.;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WF2ContrastChange(TObject *Sender)
{
    wf2gamma = (float)WF2Contrast->Position / 100.;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SP1FSpeedClick(TObject *Sender)
{
    sp1fullspeed = !sp1fullspeed;
    SP1FSpeed->Down = sp1fullspeed;

    SP1FSpeed->DrawColor = sp1fullspeed ? clLight : clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Poly1MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if(Poly1->Down) return;

	Poly1->Down = true;
	Poly2->Down = false;
    Poly4->Down = false;

    polyfactor = 1;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Poly2MouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if(Poly2->Down) return;

    Poly2->Down = true;
    Poly1->Down = false;
	Poly4->Down = false;

    polyfactor = 2;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Poly4MouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
    if(Poly4->Down) return;

    Poly4->Down = true;
    Poly1->Down = false;
    Poly2->Down = false;

    polyfactor = 4;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::bIsWindowsXP(void)
{
	   OSVERSIONINFO osvi;

// Initialize the OSVERSIONINFO structure.//
	   ZeroMemory(&osvi, sizeof(OSVERSIONINFO));

	   osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	   GetVersionEx((OSVERSIONINFO*)&osvi);

	   isXPorGT =
		(osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) &&   // excludes 98, ME, 95
		((osvi.dwMajorVersion > 5) ||                     // includes Vista
		 ((osvi.dwMajorVersion == 5) &&
		 ((osvi.dwMinorVersion == 1) || (osvi.dwMinorVersion == 2))));//includes XP & 2003

	   return isXPorGT;
}
//---------------------------------------------------------------------------
bool  __fastcall TMainForm::getCpuLoad(int &cpuTot, int &cpuProc)
{
   __int64 usr, kern, idle, sys, dusr, dkern, didle, crea, exit, pkern, puser,
		   dpkern, dpuser;
   static __int64 last_usr, last_kern, last_idle, last_pkern, last_puser;
   static bool first = true;
   bool res;

   if(!GSTaddr) return false;

   res =  GSTaddr(&idle, &kern, &usr);
   res &= GetProcessTimes(GetCurrentProcess(), (FILETIME*)&crea,
						 (FILETIME*)&exit, (FILETIME*)&pkern, (FILETIME*)&puser);

   if(!res) return false;

   if(first)
   {
	 last_usr   = usr;
	 last_kern  = kern;
	 last_idle  = idle;
	 last_puser = puser;
	 last_pkern = pkern;
	 first = false;
	 return false;
   }

   dusr   = usr  - last_usr;     last_usr  = usr;
   dkern  = kern - last_kern;    last_kern = kern;
   didle  = idle - last_idle;    last_idle = idle;
   dpkern = pkern - last_pkern;  last_pkern = pkern;
   dpuser = puser - last_puser;  last_puser = puser;

   sys = dusr + dkern;
   if(sys == 0) return false;
   cpuTot  = (int)(((float)sys - didle) * 100. / sys + 0.5);
   cpuProc = (int)((float)(dpkern + dpuser) * 100. / sys + 0.5);
   return true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BWF1RevClick(TObject *Sender)
{
   revWF1scroll = !revWF1scroll;
   BWF1Rev->Down = revWF1scroll;

   BWF1Rev->DrawColor = revWF1scroll ? clLight : clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BWF1AvgClick(TObject *Sender)
{
   WF1avg = !WF1avg;
   BWF1Avg->Down = WF1avg;

   BWF1Avg->DrawColor = WF1avg ? clLight : clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BWF2AvgClick(TObject *Sender)
{
   WF2avg = !WF2avg;
   BWF2Avg->Down = WF2avg;

   BWF2Avg->DrawColor = WF2avg ? clLight : clGray;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelExtHWClick(TObject *Sender)
{
   bool wasrunning = running;

   if(controllingExtHW)
   {
     if(ShowGUI != NULL) ShowGUI();
	 SelExtHW->Checked = true;
	 return;
   }

   if(wavin)
     WStopClick(Sender);
   else
	 if(running) StartStopClick(Sender);

   if(!extHWready)
   {
	 extHWready = InitHW(extHWname, extHWmodel, (int &)extHWtype);
     SetCallback(extIOCallback);
     SelExtHW->Caption = extHWname;
   }

   if(!extHWready)     // external HW did not come ready
   {
	 SelSoundCardClick(Sender);
	 return;
   }

   if(!OpenHW()) return;

   controllingExtHW = true;
   SetHWLO(extLOfreq);
   
   if(TuneChanged != NULL)
      TuneChanged((long)(FF1freq + extLOfreq + 0.5));

   extio = true;  // suppose the external hardware has its own digitizer

   if(extHWtype == SDRX || extHWtype == SCdata)
   { extio = false;      // SDRX and SCdata just use the sound card
	 return;
   }

// in case extio is true, we must set some things...

   MASIO->Visible = false;
   MWMMEClick(Sender);

   srateIn = GetHWSR();
   OtherRate->Caption = srateIn;
   SOtherInClick(Sender);
   oddRate = true;

   SetParam(Sender);
   InRates->Enabled = false;
   if(wasrunning) StartStopClick(Sender);
   SelExtHW->Checked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelSoundCardClick(TObject *Sender)
{
	if(!SelSoundCard->Checked)   // it was already checked, but clicking on it made it unchecked,
	{                            // so just ignore the click, after re-checking it
	  SelSoundCard->Checked = true;
	  return;
	}

	if(running) StartStopClick(Sender);
	WPanel->Visible = false;
	WavName->Visible = false;
	WavTime->Visible = false;
	StartStop->Visible = true;
	MASIO->Visible = true;
	if(wavin) IWav->InClose();
	wavin = false;
	extio = false;
	if(controllingExtHW && NULL != HideGUI) HideGUI();
	controllingExtHW = false;
	InRates->Enabled = true;
	if(oddRate) S48000InClick(Sender);
	oddRate = false;
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::GetDLLname(char *dllname)
{
   TSearchRec sr;
   char s[256];

   if(FindFirst("ExtIO_*.dll", faAnyFile, sr) == 0)
   {
	 strcpy(dllname, sr.Name.c_str());
	 if(FindNext(sr) == 0)               // meaning more than one DLL
	 {
		FindClose(sr);
		if(!DLLOpenDialog->Execute())
		  return false;
        strcpy(dllname, DLLOpenDialog->FileName.c_str());               
	 }
	 FindClose(sr);
	 if(NULL != strstr(dllname, "perseus") || NULL != strstr(dllname, "Perseus"))
	   perseus = true;
	 return true;
   }

   FindClose(sr);
   return false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AvgWF1KeyPress(TObject *Sender, char &Key)
{
    MainForm->FormKeyPress(Sender, Key);
    Key = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AvgWF2KeyPress(TObject *Sender, char &Key)
{
	MainForm->FormKeyPress(Sender, Key);
	Key = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormKeyPress(TObject *Sender, char &Key)
{
   float Tfreq;
   String nLO, nTune;

   switch(Key)
   {
	   case 'z'   :
	   case 'Z'   : BZAPClick(Sender);  break;

	   case 'c'   : // center the received frequency
	   case 'C'   : if(wavin || LOlocked) break;
					extLOfreq += FF1freq;  FF1freq = 0; goto setLOfreq;

	   case 'h'   :
	   case 'H'   :
					if(controllingExtHW && ShowGUI != NULL)
					  ShowGUI();
					break;

	   case 0x0f  : /*Ctrl-O => new LO value via keyboard*/
					if(wavin || LOlocked) break;
					nLO =
					InputBox("digiRadio - New LO value", "Please enter the new LO frequency, in kHz",
							 String(extLOfreq/1000.));
againL:
					try
					{ extLOfreq = nLO.ToDouble() * 1000.; }
					catch(...)
					{
					  nLO =
					  InputBox("digiRadio - New LO value", "Invalid value, please re-enter or press ESC",
								String(extLOfreq/1000.));
					  goto againL;
					}

					if(nLO < 0 || nLO > 99999.999)
					{
					  nLO =
					  InputBox("digiRadio - New LO value", "Value outside the limits, " \
							   "please re-enter or press ESC",
								String(extLOfreq/1000.));
					  goto againL;
					}
setLOfreq:
					if(controllingExtHW)
					  SetHWLO((long)extLOfreq);

					SetLONixieFreq(extLOfreq);
					for(int k=0; k<NUMDIGITS; k++)  ExtLoDigit[k].Show(ExtLoDig[k]);
					FF1Scale->StartVal = (FF1InizioScala + extLOfreq) / 1000.;
					FF1Scale->StopVal  = FF1Scale->StartVal + FF1span / 1000.;
					FF1Scale->Repaint();

					if(controllingExtHW && IFLimitsChanged != NULL)
					   IFLimitsChanged(FF1InizioScala + extLOfreq,
					   FF1InizioScala + extLOfreq + FF1span);

					oldLO = extLOfreq;
					AdjustBlueWindow();
					break;

	   case 0x14  : /*Ctrl-T => new Tune value via keyboard*/
					nTune =
					InputBox("digiRadio - New Tune value", "Please enter the new Tune frequency, in kHz",
							 String((FF1freq + extLOfreq)/1000.));
againT:
					try
					{ Tfreq = nTune.ToDouble() * 1000.; }
					catch(...)
					{
					  nTune =
					  InputBox("digiRadio - New Tune value", "Invalid value, please re-enter or press ESC",
								String((FF1freq + extLOfreq)/1000.));
					  goto againT;
					}

					if((Tfreq - extLOfreq) < lobound || (Tfreq - extLOfreq) > hibound)
					{
					  nTune =
					  InputBox("digiRadio - New Tune value", "Value outside the limits, " \
							   "please re-enter or press ESC",
								String((FF1freq + extLOfreq)/1000.));
					  goto againT;
					}

					FF1freq = Tfreq - extLOfreq;
					if(GetNCONixieFreq() != Tfreq)
					{
					  SetNCONixieFreq(Tfreq);
					  for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
					}
					AdjustBlueWindow();
					break;

	   case 0x01  : /* Ctrl-A => Mode AM */
					BAMClick(Sender);
					break;

	   case 0x05  : /* Ctrl-E => Mode ECSS */
					BECSSClick(Sender);
					break;

	   case 0x06  : /* Ctrl-F => Mode FM */
					BFMClick(Sender);
					break;

	   case 0x0c  : /* Ctrl-L => Mode LSB */
					BLSBClick(Sender);
					break;

	   case 0x15  : /* Ctrl-U => Mode USB */
					BUSBClick(Sender);
					break;

	   case 0x03  : /* Ctrl-C => Mode CW */
					BCWClick(Sender);
					break;

	   case 0x04  : /* Ctrl-D => Mode DRM */
					BDRMClick(Sender);
					break;

	   case 0x12  : /* Ctrl-R => Noise Reduction */
					if(NRed->Down)
					{
					  nr = false;
					  NRed->Down = false;
					}
					else
					{
					  nr = true;
					  NRed->Down = true;
					}
					break;


	   case 0x10  : /* Ctrl-P => CW Peak */
					if(CWPk->Down)
					{
					  cwp = false;
					  CWPk->Down = false;
					}
					else
					{
					  if(mode != CW) return;       // Denoiser 2 is for CW only...
					  cwp = true;
					  CWPk->Down = true;
					}
					break;

	   case 0x02  : /* Ctrl-B => NoiseBlanker */
					if(NBlanker->Down)
					{
					  nblanker = false;
					  NBlanker->Down = false;
					}
					else
					{
					  nblanker = true;
					  NBlanker->Down = true;
					}
					break;

	}


   Key = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
   CoUninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowStatusClick(TObject *Sender)
{
   FStatus->Visible = !FStatus->Visible;
   if(FStatus->Visible)
   {
	 ::SetWindowPos(FStatus->Handle, HWND_TOPMOST,
				   0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	 ShowStatus->Caption = "Hide Status";
   }
   else
	 ShowStatus->Caption = "Show Status";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AdjustBlueWindow(void)
{
   switch(mode)
   {
	 case AM   :
	 case FM   :
		  switch(submode)
		  {
			 case lower:
				rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
				rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;
				break;
			 case upper:
			 case both:
				rxband.left   = (FF1freq - wfm_rf_cut / 2 - FF1InizioScala)/FF1HzPerPixel;
				rxband.right  = (FF1freq + wfm_rf_cut / 2 - FF1InizioScala)/FF1HzPerPixel;
		  }
		  break;
	 case ECSS :
		  switch(submode)
		  {
			 case lower :  rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
						   rxband.right  = (FF1freq - FF2locut - FF1InizioScala)/FF1HzPerPixel;
						   break;
			 case upper :  rxband.left   = (FF1freq + FF2locut - FF1InizioScala)/FF1HzPerPixel;
						   rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;
						   break;
			 case both  :  rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
						   rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;
						   break;
		  }
		  break;

	 case DRM  :
	 case USB  :
		  rxband.left   = (FF1freq + FF2locut - FF1InizioScala)/FF1HzPerPixel;
		  rxband.right  = (FF1freq + FF2hicut - FF1InizioScala)/FF1HzPerPixel;
		  break;

	 case LSB  :
		  rxband.left   = (FF1freq - FF2hicut - FF1InizioScala)/FF1HzPerPixel;
		  rxband.right  = (FF1freq - FF2locut - FF1InizioScala)/FF1HzPerPixel;
          break;

     case  CW  :
          rxband.left   = (FF1freq - (FF2hicut - FF2locut)/2 - FF1InizioScala)/FF1HzPerPixel;
          rxband.right  = (FF1freq + (FF2hicut - FF2locut)/2 - FF1InizioScala)/FF1HzPerPixel;
          break;
   }
   if(NULL != FiltersChanged && !mustGetFilters)
	  FiltersChanged((int)(FF2locut + 0.5F), (int)(FF2hicut + 0.5F), (int)(CWPitch + 0.5F));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DigitMouseDown(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
   TuneTimer->Enabled = true;
   tuneUp = Button == mbLeft;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DigitMouseUp(TObject *Sender,
	  TMouseButton Button, TShiftState Shift, int X, int Y)
{
   TuneTimer->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetLONixieFreq(int freq)
{
	freq += extLOoffset;
	int div = pow(10, NUMDIGITS-1) + 0.5, n;

	for(int k = 0; k < NUMDIGITS; k++)
	{ n = freq / div;
	  ExtLoDigit[k].value = n;
	  freq -= n * div;
	  div /= 10;
	}
}
//---------------------------------------------------------------------------
long __fastcall TMainForm::GetLONixieFreq(void)
{
	long valtot = 0;

	for(int k = 0; k < NUMDIGITS; k++)
	  valtot = valtot * 10 + ExtLoDigit[k].value;

	return valtot - extLOoffset;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetNCONixieFreq(int freq)
{
	freq += extLOoffset;
	int div = pow(10, NUMDIGITS-1) + 0.5, n;

    if(controllingExtHW && TuneChanged != NULL)
	  TuneChanged(freq);

	NCODigit[0].negative = false;      // used 0 as index, but any would do
    for(int k = 0; k < NUMDIGITS; k++)
    { n = freq / div;
	  NCODigit[k].SetVal(n);
	  freq -= n * div;
      div /= 10;
    }
}
//---------------------------------------------------------------------------
long __fastcall TMainForm::GetNCONixieFreq(void)
{
    long valtot = 0;

    for(int k = 0; k < NUMDIGITS; k++)
      valtot = valtot * 10 + NCODigit[k].value;

	return NCODigit[0].negative ?  (extLOoffset - valtot) : (valtot - extLOoffset);   // used 0 as index, but any would do
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TuneTimerTimer(TObject *Sender)
{
    int  idx;

    if(!inLO && !inNCO)
    {
      TuneTimer->Enabled = false;
      return;
    }

    if(inLO && (LOlocked || wavin))  return;

	for(int k = 0; k < NUMDIGITS; k++)
	{
	  if(inExtLoDig[k] || inNCODig[k]) idx = k;
	}

	if(tuneUp)                   // left mouse button
		 if(inLO) ExtLoDigit[idx].Inc();
		 else     NCODigit[idx].Inc();
	else                         // right mouse button
		 if(inLO) ExtLoDigit[idx].Dec();
		 else     NCODigit[idx].Dec();

	if(inLO)
	{
	  extLOfreq = GetLONixieFreq();

      if(controllingExtHW)
      {
        SetHWLO((long)extLOfreq);
	  }

	  FF1Scale->StartVal = (FF1InizioScala + extLOfreq + extLOoffset) / 1000.;
	    FF1Scale->StopVal  = FF1Scale->StartVal + FF1span / 1000.;
      FF1Scale->Repaint();

	    if(controllingExtHW && IFLimitsChanged != NULL)
	      IFLimitsChanged(FF1InizioScala + extLOfreq, FF1InizioScala + extLOfreq + FF1span);

      if(extLOfreq != oldLO && controlKey)
      {
        SetNCONixieFreq(FF1freq + oldLO);
        oldLO = extLOfreq;
        goto update_NCO;
      }
	  oldLO = extLOfreq;
    }
    else
    {
update_NCO:
      FF1freq = GetNCONixieFreq() - extLOfreq;

      if(FF1freq > hibound)
      {
        FF1freq = hibound;
		SetNCONixieFreq(FF1freq + extLOfreq);
        for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
      }

      if(FF1freq < lobound)
	  {
        FF1freq = lobound;
        SetNCONixieFreq(FF1freq + extLOfreq);
        for(int k=0; k<NUMDIGITS; k++)  NCODigit[k].Show(NCODig[k]);
	  }

	  AdjustBlueWindow();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowASIOCPClick(TObject *Sender)
{
	if(driver != ASIO || !*asioName || *asioName == ' ')
	   return;

	if(running)
	  StartStopClick(Sender);

	sampleRate = (float)srateIn;
	InitializeASIO(true);  // true => dummy request, not a real start

	ASIOControlPanel();

	DisposeASIO();

	InitializeASIO(true);  // true => dummy request, not a real start
	DisposeASIO();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S8000OutClick(TObject *Sender)
{
	if(S8000Out->Down) return;
	if(!driver == ASIO) srateOut = 8000;
	S8000Out->Down  = true;
	S11025Out->Down = false;
	S24000Out->Down = false;
	S48000Out->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S11025OutClick(TObject *Sender)
{
	if(S11025Out->Down) return;
	if(!driver == ASIO) srateOut = 11025;
	S8000Out->Down  = false;
	S11025Out->Down = true;
	S24000Out->Down = false;
	S48000Out->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S24000OutClick(TObject *Sender)
{
	if(S24000Out->Down) return;
	if(!driver == ASIO) srateOut = 24000;
	S8000Out->Down  = false;
	S11025Out->Down = false;
	S24000Out->Down = true;
	S48000Out->Down = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::S48000OutClick(TObject *Sender)
{
	if(S48000Out->Down) return;
	if(!driver == ASIO) srateOut = 48000;
	S8000Out->Down  = false;
	S11025Out->Down = false;
	S24000Out->Down = false;
	S48000Out->Down = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LockVolClick(TObject *Sender)
{
	if(LockVol->Checked)
	  LockVol->Checked = vollocked = false;
	else
	  LockVol->Checked = vollocked = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MPrioNormClick(TObject *Sender)
{
	if(MPrioNorm->Checked) return;

	processPriority = NORMAL_PRIORITY_CLASS;

    if(running)
	{
	  StartStopClick(Sender);
	  Sleep(100);
	  StartStopClick(Sender);
    }

    MPrioNorm->Checked = true;
    MPrioHi->Checked = false;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MPrioHiClick(TObject *Sender)
{
	if(MPrioHi->Checked) return;

    processPriority = HIGH_PRIORITY_CLASS;

    if(running)
    {
	  StartStopClick(Sender);
	  Sleep(100);
	  StartStopClick(Sender);
    }

    MPrioNorm->Checked = false;
    MPrioHi->Checked = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
    if(Shift.Contains(ssCtrl))
      controlKey = true;

    if(Shift.Contains(ssAlt))
      altKey = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FormKeyUp(TObject *Sender, WORD &Key,
	  TShiftState Shift)
{
	controlKey = altKey = false;
}
//---------------------------------------------------------------------------
void  __fastcall TMainForm::StartClk(void)
{
	QueryPerformanceCounter(&startTime);
}
//---------------------------------------------------------------------------
void  __fastcall TMainForm::StopClk(void)
{
	QueryPerformanceCounter(&stopTime);
	float usec = (stopTime.QuadPart - startTime.QuadPart) * 1.e6
												/ TicksPerSecond.QuadPart;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BDespreadClick(TObject *Sender)
{
	static bool guard = false;

	if(guard)
	{
	  guard = false;
	  return;
	}

	if(mode != CW) return;  // despreading is for CW only

	guard= true;
	BDespread->Down = !despreading;
	cwp = !despreading;
	CWPk->Down = !despreading;
	despreading = !despreading;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BECSSMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift,
	  int X, int Y)
{
	if(mode == ECSS)
	{
	  switch(submode)
	  {
		case lower :  submode = upper; break;
		case upper :  submode = both;  break;
		case both  :  submode = lower; break;
	  }

	  AdjustBlueWindow();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WTrackMouseDown(TObject *Sender, TMouseButton Button, TShiftState Shift,
	  int X, int Y)
{
	 IWav->InPosition((float)X / WTrack->Width);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::BMuteClick(TObject *Sender)
{
	audiomute = !audiomute;
	BMute->Down = audiomute;
}
//---------------------------------------------------------------------------
float __fastcall inline TMainForm::getdBfromy(int y)
{
	return y * (float)(SP1bottomrange - SP1toprange) / SP1rect.Bottom + SP1toprange;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayWavTime(void)
{
	char s[64];

	if(perseuswav || rfspacewav || winradwav)
	  sprintf(s, "%s %d, %d - %02d:%02d:%02dZ", Month[IWav->wM], IWav->wd, IWav->wy,
											    IWav->wh, IWav->wm, IWav->ws);
	else
	  sprintf(s, "%02d:%02d:%02d", IWav->wh, IWav->wm, IWav->ws);

	WavTime->Caption = s;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayRecTime(void)
{
	char s[64];

	sprintf(s, "%5.3f GB", (float)owavsize / 1.e9F);

	RecGB->Caption = s;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WRecClick(TObject *Sender)
{
	long hi = FindClassHInstance(__classid(TMainForm));
	AnsiString fname;

	switch (wrec)
	{
		case wstopped:
		  fname = GenerateTimeStamp(".wav");
		  if(!OWav->OutOpen(fname.c_str()))
		  {
            warn("Unable to create a new output WAV file"); 
			return;
		  }

		  WRec->Glyph->LoadFromResourceName(hi, "RECORD_ON");
		  LRec->Visible = true;
		  LOLock->Visible = LOlocked = true;
		  RecGB->Visible = true;
		  RecPanel->Width = 101;   // make visible the pause button
		  wrec = wrecording;
		  break;

		case wpaused: break;

		case wrecording:
		  WRec->Glyph->LoadFromResourceName(hi, "RECORD_OFF");
		  LRec->Visible = false;
		  LOLock->Visible = LOlocked = false;
		  RecGB->Visible = false;
		  RecPanel->Width = 61;   // hide the pause button
		  wrec = wstopped;
		  OWav->OutClose("");
		  break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WRecPauseClick(TObject *Sender)
{
	long hi = FindClassHInstance(__classid(TMainForm));

	switch (wrec)
	{
		case wrecording:
		  WRec->Glyph->LoadFromResourceName(hi, "RECORD_PAUSED");
		  LRec->Caption = "Paused";
		  wrec = wpaused;
		  break;

		case wpaused:
		  WRec->Glyph->LoadFromResourceName(hi, "RECORD_ON");
		  LRec->Caption = "Recording";
		  wrec = wrecording;
		  break;
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::GenerateTimeStamp(AnsiString ext)
{
	int p;
	AnsiString freq = String((int)(extLOfreq/1000. + 0.5));
	char s[256];
	time_t t;
	struct tm *gmt;

	t = time(NULL);
	gmt = gmtime(&t);

	sprintf(s, "Winrad_%04d%02d%02d_%02d%02d%02dZ_%skHz",
			   gmt->tm_year + 1900,
			   gmt->tm_mon + 1,
			   gmt->tm_mday,
			   gmt->tm_hour,
			   gmt->tm_min,
			   gmt->tm_sec,
			   freq.c_str());

	AnsiString at = ExtractFilePath(Application->ExeName) + s + ext;

	return at;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComboBox1Change(TObject *Sender)
{
	   wfm_deemph_tau = atof(String(ComboBox1->Text).c_str()) / 1.e6;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComboBox2Change(TObject *Sender)
{
	   wfm_rf_cut = atoi(String(ComboBox2->Text).c_str()) * 1000;
	   if(wfm_rf_cut > srateIn)
	   {
		  wfm_rf_cut = srateIn;
	   }
	   AdjustBlueWindow();

}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComboBox3Change(TObject *Sender)
{
	   wfm_audio_cut = atoi(String(ComboBox3->Text).c_str()) * 1000;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ComboBox4Change(TObject *Sender)
{
	extLOoffset = atoi(String(ComboBox4->Text).c_str()) * 1000;
	SetNCONixieFreq(extLOfreq + FF1freq);
	SetParam(this);
}


